using System;
using System.Collections;

namespace CbModel
{
	/// <summary>
	/// Summary description for ContactList.
	/// </summary>
	public class ContactList
	{
		GroupLab.Networking.SharedDictionary sd;
		GroupLab.Networking.Subscription contactSub;

		MainModel mainModel;
		Owner owner;
		string key;

		private Hashtable contacts;

		Item presence;

		public ContactList(GroupLab.Networking.SharedDictionary sd, MainModel mainModel, Owner owner, string key)
		{
			this.sd = sd;
			this.mainModel = mainModel;
			this.owner = owner;
			this.key = key;
			this.contacts = new Hashtable();

			this.presence = new Item(this.sd, this.mainModel, this.Key);
		}

		public string Key
		{
			get
			{
				return this.key;
			}
		}

		public string Name
		{
			get
			{
				return (string)((GroupLab.Networking.Map)this.sd[this.Key])["name"];
			}
		}

		/// <summary>
		/// Remove all the keys for this contact list from the dictionary
		/// </summary>
		public void Delete()
		{
			this.sd.Remove(this.Key);
		}

		public event ContactListEventHandler Activated;
		protected virtual void OnActivate(ContactListEventArgs e)
		{
			if(null != this.Activated)
			{
				this.Activated(this, e);
			}
		}
		public event PersonEventHandler ContactAdded;
		protected virtual void OnContactAdded(PersonEventArgs e)
		{
			if(null != this.ContactAdded)
			{
				this.ContactAdded(this, e);
			}
		}
		public event PersonEventHandler ContactRemoved;
		protected virtual void OnContactRemoved(PersonEventArgs e)
		{
			if(null != this.ContactRemoved)
			{
				this.ContactAdded(this, e);
			}
		}
		public event PersonEventHandler ContactLoggedIn;
		protected virtual void OnContactLoggedIn(PersonEventArgs e)
		{
			if(null != this.ContactLoggedIn)
			{
				this.ContactLoggedIn(this, e);
			}
		}
		public event PersonEventHandler ContactLoggedOut;
		protected virtual void OnContactLoggedOut(PersonEventArgs e)
		{
			if(null != this.ContactLoggedOut)
			{
				this.ContactLoggedOut(this, e);
			}
		}

		public void Activate()
		{
			// set up the subscriptions
			this.contactSub = new GroupLab.Networking.Subscription();
			this.contactSub.BeginInit();
			this.contactSub.SubscriptionChanged += new GroupLab.Networking.SubscriptionChangedEventHandler(this.contactSub_SubscriptionChanged);
			this.contactSub.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.contactSub_Notified);
			this.contactSub.Enabled = true;
			string pattern = this.owner.Key+"/users/?|value is `"+this.key+"`";
			//Console.WriteLine(pattern);
			this.contactSub.Pattern = pattern;
			this.contactSub.Dictionary = this.sd;
			this.contactSub.EndInit();
		}

		public Item Presence
		{
			get
			{
				return this.presence;
			}
		}

		public void AddContact(Person person)
		{
			string key = this.owner.Key+person.Key;
			this.sd[key] = this.Key;
		}

		public void RemoveContact(Person person)
		{
			this.sd.Remove(this.owner.Key+person.Key);
		}

		private void contactSub_SubscriptionChanged(object sender, GroupLab.Networking.SubscriptionChangedEventArgs e)
		{
			if(e.Reason != GroupLab.Networking.SubscriptionNotification.Remove)
			{
				ContactListEventArgs cle = new ContactListEventArgs(this);
				this.OnActivate(cle);
			}
		}

		private void contactSub_Notified(object sender, GroupLab.Networking.SubscriptionEventArgs e)
		{
			// work out the contact key
			int startindex = this.mainModel.Owner.Key.Length;
			string val = e.Path as string;
			string contactKey = val.Substring(startindex);
			Person contact = new Person(this.sd, this.mainModel, contactKey);
			PersonEventArgs pe = new PersonEventArgs(contact);
			contact.Activate();
			contact.StatusChanged += new PersonEventHandler(this.contact_StatusChanged);
			if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason)
			{
				// new contact for the group added
				this.contacts.Add(contactKey, contact);
				this.OnContactAdded(pe);
			}
			else if(GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
			{
				// contact removed
				this.contacts.Remove(contactKey);
				this.OnContactRemoved(pe);
			}
			this.contact_StatusChanged(sender, pe);
		}

		public ICollection Contacts
		{
			get
			{
				return this.contacts.Values;
			}
		}

		public bool IsContact(Person p)
		{
			bool present = false;
			foreach(Person contact in this.contacts.Values)
			{
				if(contact == p)
				{
					present = true;
					break;
				}
			}
			return present;
		}

		public static bool operator != (ContactList a, ContactList b)
		{
			if(object.ReferenceEquals(a, null))
			{
				if(object.ReferenceEquals(b, null))
				{
					return false;
				}
				else
				{
					return true;
				}
			}
			else if(object.ReferenceEquals(b, null))
			{
				return true;
			}
			else
			{
				return (0 != string.Compare(a.Key, b.Key, true));
			}
		}		
		public static bool operator == (ContactList a, ContactList b)
		{
			if(object.ReferenceEquals(a, null))
			{
				if(object.ReferenceEquals(b, null))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else if(object.ReferenceEquals(b, null))
			{
				return false;
			}
			else
			{
				return (0 == string.Compare(a.Key, b.Key, true));
			}
		}

		public override bool Equals(object obj)
		{
			return this.GetHashCode() == obj.GetHashCode();
		}

		public override int GetHashCode()
		{
			return this.Key.GetHashCode();
		}

		private void contact_StatusChanged(object sender, PersonEventArgs e)
		{
			if(e.Person.Status == "Online")
			{
				this.OnContactLoggedIn(e);
			}
			else
			{
				this.OnContactLoggedOut(e);
			}
		}
	}
}
