using System;
using SideItem;
using System.Windows.Forms;
using GroupLab.Networking;
using System.Drawing;

namespace CbModel
{
	/// <summary>
	/// The item to show when the item type asked for is not available.
	/// </summary>
	public class DefaultItem : SideItem.IItem
	{
		GroupLab.Networking.SharedDictionary sd;

		string key;

		bool master;

		public DefaultItem()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		/// <summary>
		/// Initialise the item.
		/// </summary>
		/// <param name="sd">The shared dictionary client instance for storing all the 
		/// shared data for this item.</param>
		/// <param name="path">The unique key to identify this item in the shared dictionary.</param>
		/// <param name="master">True if this item instance is the original one created.</param>
		public void StartItem(GroupLab.Networking.SharedDictionary sd, string path, bool master, string currentUserKey)
		{
			this.sd = sd;
			this.key = path;
			// is this really necessary? the default item will never be master (I think)
			this.master = master;
		}

		/// <summary>
		/// Stop the item running.  Close down all running processes and clean up the 
		/// shared dictionary.
		/// </summary>
		public void StopItem()
		{ }

		/// <summary>
		/// Return a display for the tile view of this item at a particular awareness.
		/// </summary>
		/// <param name="size">The maximum size that the application will allow for this item.</param>
		/// <returns>A control that displays this item.</returns>
		public System.Windows.Forms.Control GetTile(int focus)
		{
			Label tile = new Label();
			tile.Text = "Unknown Item Type";
			tile.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			tile.BackColor = System.Drawing.Color.Blue;
			tile.ForeColor = System.Drawing.Color.Yellow;
			tile.Height = focus;

			return tile;
		}

		/// <summary>
		/// Return a display for the transient focus view of this item at a particular awareness.
		/// </summary>
		/// <param name="size">The maximum size that the application allows for this view.</param>
		/// <returns>A control that displays this item.</returns>
		public System.Windows.Forms.Control GetTransientFocus(Size size)
		{
			Label focus = new Label();
			string ownerKey = (string)((GroupLab.Networking.Map)this.sd[this.key])["owner"];
			string ownerName = (string)this.sd[ownerKey];
			string itemType = (string)((GroupLab.Networking.Map)this.sd[this.key])["type"];
			focus.Text = "Item type not installed.\n Type: "+itemType+"\nPosted by: "+ownerName;
			focus.TextAlign = System.Drawing.ContentAlignment.TopLeft;
			focus.Size = size;

			return focus;
		}

		/// <summary>
		/// A handle for when the user wishes to separate this item from the application.  
		/// The result of calling this method should be a sparate form or application that allows
		/// the user full interaction.
		/// </summary>
		public void SeparateItem()
		{ }

		/// <summary>
		/// Returns the maximum height on the bar that the item wants to use.
		/// 
		/// The maximum is at the moment it is requested.  There is no need for the 
		/// value to remain constant through the life of the item.  It will most
		/// likely change as the width of the bar changes.
		/// </summary>
		/// <returns>Height in pixels.</returns>
		public int MaxHeight()
		{
			return 100;
		}

		/// <summary>
		/// Returns the minimum height on the bar that the item needs to display itself.
		/// 
		/// The return value is for the moment requested.  There is no requirement for the
		/// value to remian constant through the lifetime of the item.
		/// </summary>
		/// <returns>Height in pixels.</returns>
		public int MinHeight()
		{
			return 10;
		}

		/// <summary>
		/// Tells the item the width of the bar.
		/// 
		/// The item needs to know the width that it has available to display in to
		/// set the max and min heights.  This gives the item control over its display
		/// proportions.
		/// </summary>
		/// <param name="width">The width available to the item.</param>
		public void SetWidth(int width)
		{ }

		public event SideItem.ChangeEventHandler ItemChanged;

		private void OnDefaultChanged(SideItem.ChangeEventArgs e)
		{
			if(null != this.ItemChanged)
			{
				this.ItemChanged(this, e);
			}
		}


	}
}
