using System;
using SideItem;
using GroupLab.Networking;
//using Collabrary;
using System.Collections;

namespace CbModel
{
	/// <summary>
	/// Summary description for Item.
	/// </summary>
	public class Item
	{
		GroupLab.Networking.SharedDictionary sd;

		private string key;
		private string type;

		private MainModel mainModel;

		private object tag;

		private SideItem.IItem item;

		private bool master;

		private string deletePath;
		private GroupLab.Networking.Subscription deleteSub;

		// rates how "important" this item is currently according to the 
		// changes - rating and frequency
		int importance;
		//System.Windows.Forms.Timer timer;

		//private GroupLab.Networking.Subscription infoSub;

		public Item(GroupLab.Networking.SharedDictionary sd, MainModel mainModel, string key)
		{
			this.sd = sd;
			this.mainModel = mainModel;
			this.key = key;
			Object o = this.sd[this.Key];
			GroupLab.Networking.Map map = (GroupLab.Networking.Map)o;
			Object o1 = map["owner"];
			string itemOwnerKey = (string)o1;
			this.type = (string)map["type"];
			//string itemOwnerKey = (string)((GroupLab.Networking.Map)this.sd[this.Key])["owner"];
			this.master = (this.mainModel.Owner.Key == itemOwnerKey);

			this.importance = 100;

			ItemFactory factory = ItemFactory.GetItemFactory(this.sd, this.mainModel.Owner);
			this.item = factory.GetExistingItem(this.key, this.mainModel.Owner);

			this.deletePath = "/delete_item/"+this.Key;
			this.deleteSub = new GroupLab.Networking.Subscription();
			this.deleteSub.BeginInit();
			this.deleteSub.Dictionary = this.sd;
			this.deleteSub.Pattern = this.deletePath;
			this.deleteSub.Notified += new SubscriptionEventHandler(this.deleteSub_Notified);
			this.deleteSub.EndInit();

			//this.item.ItemChanged += new ChangeEventHandler(this.item_ItemChanged);

//			this.timer = new System.Windows.Forms.Timer();
//			this.timer.Tick += new EventHandler(this.timer_Tick);
//			this.timer.Interval = 500;
//			this.timer.Start();
		}

		// NEW METHOD - 25.07.2005
		public string Type
		{
			get
			{
				return this.type;
			}

		}

		public Place Place
		{
			get
			{
				string placeKey = (string)((GroupLab.Networking.Map)this.sd[this.Key])["place"];
				return this.mainModel.GetPlace(placeKey);
			}
		}

		public bool Master
		{
			get
			{
				return this.master;
			}
		}

		public Person Owner
		{
			get
			{
				if(this.sd.Contains(this.Key))
				{
					string personKey = (string)((GroupLab.Networking.Map)this.sd[this.Key])["owner"];
					return this.mainModel.GetPerson(personKey);
				}
				else
				{
					return null;
				}
			}
		}
		public string Key
		{
			get
			{
				return this.key;
			}
		}
		public object Tag
		{
			get
			{
				return this.tag;
			}
			set
			{
				this.tag = value;
			}
		}

		public void Delete()
		{
			this.sd[this.deletePath] = "delete";
			this.sd.Remove(this.deletePath);
		}

		public override bool Equals(object obj)
		{
			return this.GetHashCode() == obj.GetHashCode();
		}

		public override int GetHashCode()
		{
			return this.Key.GetHashCode();
		}


		#region IItem Members

		public void SeparateItem()
		{
			this.item.SeparateItem();
		}

		public System.Windows.Forms.Control GetTransientFocus(System.Drawing.Size size)
		{
			// TODO:  Add Item.GetTransientFocus implementation
			return this.item.GetTransientFocus(size);
		}

		public void StopItem()
		{
			
			this.item.StopItem();
		}

		public System.Windows.Forms.Control GetTile(int focus)
		{
			return this.item.GetTile(focus);
		}

		public void StartItem(GroupLab.Networking.SharedDictionary sd, string path, bool master)
		{
			this.item.StartItem(this.sd, this.key, this.master, this.mainModel.Owner.Key);
		}

		#endregion


		public static bool operator != (Item a, Item b)
		{
			if(object.ReferenceEquals(a, null))
			{
				if(object.ReferenceEquals(b, null))
				{
					return false;
				}
				else
				{
					return true;
				}
			}
			else if(object.ReferenceEquals(b, null))
			{
				return true;
			}
			else
			{
				return (0 != string.Compare(a.Key, b.Key, true));
			}
		}		
		public static bool operator == (Item a, Item b)
		{
			if(object.ReferenceEquals(a, null))
			{
				if(object.ReferenceEquals(b, null))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else if(object.ReferenceEquals(b, null))
			{
				return false;
			}
			else
			{
				return (0 == string.Compare(a.Key, b.Key, true));
			}
		}

		public event SideItem.ChangeEventHandler ItemChanged;

		private void OnChanged(SideItem.ChangeEventArgs e)
		{
			if(null != this.ItemChanged)
			{
				this.ItemChanged(this, e);
			}
		}
		public event ChangeEventHandler ImportanceChangedUp;
		private void OnImportanceChangedUp(ChangeEventArgs e)
		{
			if(null != this.ImportanceChangedUp)
			{
				this.ImportanceChangedUp(this, e);
			}
		}
		public event ChangeEventHandler ImportanceChangedDown;
		private void OnImportanceChangedDown(ChangeEventArgs e)
		{
			if(null != this.ImportanceChangedDown)
			{
				this.ImportanceChangedDown(this, e);
			}
		}

		public int Importance
		{
			get
			{
				return this.importance;
			}
			set
			{
				bool up = (value > this.importance);
				this.importance = value;
				ChangeEventArgs e = new ChangeEventArgs(this.importance);
				if(up)
				{
					this.OnImportanceChangedUp(e);
				}
				else
				{
					this.OnImportanceChangedDown(e);
				}
			}
		}

		private void item_ItemChanged(object sender, ChangeEventArgs e)
		{
			this.OnImportanceChangedUp(e);
			this.OnChanged(e);
		}

		private void timer_Tick(object sender, EventArgs e)
		{
			this.OnImportanceChangedDown(new ChangeEventArgs(1));
		}

		public override string ToString()
		{
			string name = base.ToString();
			if(this.sd.Contains(this.Key))
			{
				GroupLab.Networking.Map m = this.sd[this.Key] as GroupLab.Networking.Map;
				if(null != m)
				{
					string type = m["type"] as string;
					if(null != type)
					{
						name = type;
					}
				}
			}
			return name;
		}

		private void deleteSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason)
			{
				this.StopItem();
				if(this.sd.Contains(this.Key+"*"))
				{
					this.sd.Remove(this.Key+"*");
				}
			}
		}

		// For logging purposes - NEW METHOD - 26.07.2005
		public void logTooltipDisplay()
		{
			try
			{
				// for logging purposes
				string placeId = this.Place.Key.Substring(8);
				string path = "/places/" + placeId + "/tooltip_events";

				GroupLab.Networking.Map map = new Map();
				map.Add("open", "true");

				GroupLab.Networking.Map itemMap = this.sd[this.Key] as Map;	

				string itemType = itemMap["type"] as string;
				string itemId = this.Key; //this.Key.Substring(51);

				map.Add("type", itemType);
				map.Add("id", itemId);
				map.Add("person", this.mainModel.Owner.DisplayName);
				string initiatorid = this.mainModel.Owner.Key; //.Substring(7);
				map.Add("personid", initiatorid);			
				
				this.sd[path] = map;
			}
			catch (Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.StackTrace);
			}
		}

		// For logging purposes - NEW METHOD - 26.07.2005
		public void logTooltipClose() 
		{	
			try
			{
				string placeId = this.Place.Key.Substring(8);
				string path = "/places/" + placeId + "/tooltip_events";

				GroupLab.Networking.Map map = new Map();
				map.Add("open", "false");

				GroupLab.Networking.Map itemMap = this.sd[this.Key] as Map;	

				string itemType = itemMap["type"] as string;
				string itemId = this.Key; //this.Key.Substring(51);

				map.Add("type", itemType);
				map.Add("id", itemId);
				map.Add("person", this.mainModel.Owner.DisplayName);
				string initiatorid = this.mainModel.Owner.Key; //this.mainModel.Owner.Key.Substring(7);
				map.Add("personid", initiatorid);			
				
				this.sd[path] = map;
			}
			catch (Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.StackTrace);
			}
		}

		// For logging purposes - NEW METHOD - 26.07.2005
		public void logCloseItem()
		{
			try
			{
				// for logging purposes
				string placeId = this.Place.Key.Substring(8);
				string path = "/places/" + placeId + "/close_events";

				GroupLab.Networking.Map map = new Map();
				//map.Add("", focus);

				GroupLab.Networking.Map itemMap = this.sd[this.Key] as Map;	

				string itemType = itemMap["type"] as string;
				string itemId = this.Key; //this.Key.Substring(51);

				map.Add("type", itemType);
				map.Add("id", itemId);
				map.Add("person", this.mainModel.Owner.DisplayName);
				string initiatorid = this.mainModel.Owner.Key; //.Substring(7);
				map.Add("personid", initiatorid);			
				
				this.sd[path] = map;
			}
			catch (Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.StackTrace);
			}
		}

		// For logging purposes - NEW METHOD - 26.07.2005
		public void logFocusChange(string focus)
		{
			try
			{
				// for logging purposes
				string placeId = this.Place.Key.Substring(8);
				string path = "/places/" + placeId + "/focus_events";

				GroupLab.Networking.Map map = new Map();
				map.Add("focus", focus);

				GroupLab.Networking.Map itemMap = this.sd[this.Key] as Map;	

				string itemType = itemMap["type"] as string;
				string itemId = this.Key; //this.Key.Substring(51);

				map.Add("type", itemType);
				map.Add("id", itemId);
				map.Add("person", this.mainModel.Owner.DisplayName);
				string initiatorid = this.mainModel.Owner.Key; //.Substring(7);
				map.Add("personid", initiatorid);			
				
				this.sd[path] = map;
			}
			catch (Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.StackTrace);
			}
		}

		// For logging purposes - NEW METHOD - 26.07.2005
		public void logLockChange(string locked) 
		{	
			try
			{
				string placeId = this.Place.Key.Substring(8);
				string path = "/places/" + placeId + "/lock_events";

				GroupLab.Networking.Map map = new Map();
				map.Add("locked", locked);

				GroupLab.Networking.Map itemMap = this.sd[this.Key] as Map;	

				string itemType = itemMap["type"] as string;
				string itemId = this.Key; //this.Key.Substring(51);

				map.Add("type", itemType);
				map.Add("id", itemId);
				map.Add("person", this.mainModel.Owner.DisplayName);
				string initiatorid = this.mainModel.Owner.Key; //this.mainModel.Owner.Key.Substring(7);
				map.Add("personid", initiatorid);			
				
				this.sd[path] = map;
			}
			catch (Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.StackTrace);
			}
		}

	}
}
