using System;
using System.Collections;
using GroupLab.Networking;
using System.Reflection;

using SideItem;
using PresenceItem;
using InvitationItem;

namespace CbModel
{
	/// <summary>
	/// Summary description for ItemFactory.
	/// </summary>
	public class ItemFactory
	{
		private static ItemFactory factory;

		public static ItemFactory GetItemFactory(GroupLab.Networking.SharedDictionary sd, Person owner)
		{
			if (factory == null)
			{
				factory = new ItemFactory(sd, owner);
			}
			return factory;
		}
		
		// hashtable to store all the types of items
		Hashtable itemTypes;

		// link to the shared dictionary
		GroupLab.Networking.SharedDictionary sd;

		// name of the user creating all these items
		Person owner;

		string itemDirectory;

		Hashtable ownerKeyToItems;
		private System.Windows.Forms.Timer startTmr;


		public ItemFactory(GroupLab.Networking.SharedDictionary sd, Person owner)
		{
			this.itemTypes = new System.Collections.Hashtable();
			//this.sd = sd;
			//this.owner = owner;

			this.itemDirectory = System.IO.Directory.GetCurrentDirectory()+"\\items";

			this.ownerKeyToItems = new System.Collections.Hashtable();
			this.startTmr = new System.Windows.Forms.Timer();
			this.startTmr.Enabled = false;
			this.startTmr.Tick += new System.EventHandler(this.tooltipTmr_Tick);

			this.sd = sd;
			this.owner = owner;

			// intialise the dictionary
			this.LoadItems();
		}

		public SideItem.IItem GetExistingItem(string key, Person currentUser)
		{
			// get the type name for this item
			string type = (string)((GroupLab.Networking.Map)this.sd[key])["type"];

			string ownerKey = (string)((GroupLab.Networking.Map)this.sd[key])["owner"];
			bool master =  (ownerKey == currentUser.Key);
			
			return this.MakeItem(key, type, master, currentUser);
		}

		public SideItem.IItem CreateNewItem(string key, string type, Person currentUser)
		{
			return this.MakeItem(key, type, true, currentUser);
		}
		
		public SideItem.IItem MakeItem(string key, string type, bool master, Person currentUser)
		{
			System.Diagnostics.Debug.WriteLine("making an item");
			// make a new item with the type from the hashtable
//			Type t = this.itemTypes[type] as Type;
//			ConstructorInfo ci = t.GetConstructor(System.Type.EmptyTypes);
//
//			//	new Type[]{typeof(Collabrary.SharedDictionary), typeof(string), typeof(bool)});
//			SideItem.IItem item = ci.Invoke(new object[]{}) as SideItem.IItem;
//			item.StartItem(this.sd,key,master);
//
//			return item;
			SideItem.IItem item;
			if(type == "Presence")
			{
				// make a presence item
				item = new PresenceItem.PresenceItem();
				//this.sd[key+".transient"] = this.sd.Id;
			}
			else if(type == "invitation")
			{
				// make a new invitation item
				item = new InvitationItem.InvitationItem();
			}
			else if(this.itemTypes.Contains(type))
			{
				// make a new item with the type from the hashtable
				Type t = this.itemTypes[type] as Type;
				ConstructorInfo ci = t.GetConstructor(System.Type.EmptyTypes);

				//	new Type[]{typeof(Collabrary.SharedDictionary), typeof(string), typeof(bool)});
				item = ci.Invoke(new object[]{}) as SideItem.IItem;
			}
			else
			{
				// return a default item
				item = new DefaultItem();
			}
			string ownerKey = (string)((GroupLab.Networking.Map)this.sd[key])["owner"];
			if(this.sd.Contains(ownerKey))
			{
				//System.Diagnostics.Debug.WriteLine("starting normally");
				item.StartItem(this.sd,key,master,currentUser.Key);
			}
			else
			{
				this.StartTimeCheckOwner(ownerKey, key, item);
				if(!this.startTmr.Enabled)
				{
					this.startTmr.Enabled = true;
				}
			}
			return item;
		}

		private void StartTimeCheckOwner(string ownerKey, string key, SideItem.IItem item)
		{
			ArrayList itemList;
			if(this.ownerKeyToItems.Contains(ownerKey))
			{
				itemList = this.ownerKeyToItems[ownerKey] as ArrayList;
			}
			else
			{
				itemList = new ArrayList();
			}
			GroupLab.Networking.Map itemInfo = new Map();
			itemInfo["item"] = item;
			itemInfo["path"] = key;
			itemList.Add(itemInfo);
			this.ownerKeyToItems[ownerKey] = itemList;
		}

		private void tooltipTmr_Tick(object sender, System.EventArgs e)
		{
			ArrayList removeKeys = new ArrayList();
			foreach(string key in this.ownerKeyToItems.Keys)
			{
				if(this.sd.Contains(key))
				{
					removeKeys.Add(key);
					ArrayList items = this.ownerKeyToItems[key] as ArrayList;
					foreach(GroupLab.Networking.Map itemInfo in items)
					{
						System.Diagnostics.Debug.WriteLine("had to start an item");
						string path = itemInfo["path"] as string;
						SideItem.IItem item = itemInfo["item"] as SideItem.IItem;
						item.StartItem(this.sd, path, false, this.owner.Key);
					}
				}
			}
			foreach(string key in removeKeys)
			{
				this.ownerKeyToItems.Remove(key);
			}
			if(this.ownerKeyToItems.Keys.Count == 0)
			{
				this.startTmr.Enabled = false;
			}
		}


		public System.Collections.ICollection GetItemTypeNames()
		{
			ArrayList a = new ArrayList(this.itemTypes.Keys);
			a.Sort();
			a.Reverse();
			return a;
		}

		private void LoadItems()
		{
			System.Reflection.TypeFilter myFilter = new System.Reflection.TypeFilter(MyInterfaceFilter);

			// find all the dll files in the current directory
			string[] dlls = System.IO.Directory.GetFiles(this.itemDirectory, "*.dll");
			AppDomain.CurrentDomain.AppendPrivatePath(this.itemDirectory);

			foreach (string fileName in dlls)
			{
				try
				{
					// load the file
					System.Reflection.Assembly file = System.Reflection.Assembly.LoadFrom(fileName);

					// get all the types defined in the dll
					System.Type[] types = file.GetTypes();

					foreach (System.Type type in types)
					{
						// get all the interfaces implemented by this type
						System.Type[] interfaces = type.FindInterfaces(myFilter, typeof(SideItem.IItem));
						if (interfaces.Length > 0)
						{
							// it implements the interface!  
							// get the name of the item (if it has one)
							string name = "";
							object[] names = type.GetCustomAttributes(typeof(SideItem.ItemNameAttribute), false);
							if (names.Length == 0)
							{
								// no name - do the best that we can
								name = type.ToString();
							}
							else
							{
								// the type has something - use it
								name = ((SideItem.ItemNameAttribute)names[0]).Name;
							}
							// Add the reference to the dictionary.
							this.itemTypes[name] = type;
						}
					}
				}
				catch (ReflectionTypeLoadException e)
				{
					// should I do something?
					System.Diagnostics.Debug.WriteLine(e.Message);

				}
			}
		}

		public static bool MyInterfaceFilter(Type typeObj,Object criteriaObj)
		{
			if(typeObj.ToString() == criteriaObj.ToString())
				return true;
			else
				return false;
		}
	}
}
