using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;

using GroupLab.Networking;
using ColourGenerator;

namespace CbModel
{
	/// <summary>
	/// Summary description for MainModel.
	/// </summary>
	public class MainModel : System.ComponentModel.Component
	{
		// -------------------------------------------------
		// Changes the mode of operation (currently compile time option)
		private const bool contactsMode = false;
		// -------------------------------------------------

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// The shared dictionary with all the community data in it.
		/// </summary>
		private GroupLab.Networking.SharedDictionary sd;

		private PromptForLoginForm loginForm;

		// login info
		private string ownerName;
		private string ownerEmail;
		private string serverName;

		private bool deliberateStop;

		private Owner owner;

		// keep references to the people, places
		private Hashtable places;
		private Hashtable people;

		// people, place, item events
		GroupLab.Networking.Subscription placesChanged;
		GroupLab.Networking.Subscription peopleChanged;

		public MainModel(System.ComponentModel.IContainer container) : this()
		{
			///
			/// Required for Windows.Forms Class Composition Designer support
			///
			container.Add(this);
		}

		public MainModel()
		{
			///
			/// Required for Windows.Forms Class Composition Designer support
			///
			InitializeComponent();


			// subscribe to the shared dictionary events
			this.sd = new GroupLab.Networking.SharedDictionary();
			this.sd.SynchronizingObject = this.SynchronizingObject;
			this.sd.Opened += new EventHandler(this.sd_Opened);
			this.sd.Disconnected += new DisconnectedEventHandler(this.sd_Disconnected);
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			// lotsa stuff here
			base.Dispose( disposing );
		}


		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion

		/// <summary>
		/// Gets or sets an object which is used to synchronise events.
		/// </summary>
		[Description("The object used to synchronise events."), DefaultValue(null)]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if(this.DesignMode && (null == this.sd.SynchronizingObject))
				{
					System.ComponentModel.Design.IDesignerHost host = this.GetService(typeof(System.ComponentModel.Design.IDesignerHost)) as System.ComponentModel.Design.IDesignerHost;
					if(null != host)
					{
						this.sd.SynchronizingObject = host.RootComponent as ISynchronizeInvoke;
					}
				}
				return this.sd.SynchronizingObject;
			}
			set
			{
				this.sd.SynchronizingObject = value;
			}
		}

		public bool ContactsMode
		{
			get
			{
				return contactsMode;
			}
		}

		public ICollection ItemTypes
		{
			get
			{
				return (ItemFactory.GetItemFactory(this.sd, this.owner).GetItemTypeNames());
			}
		}

		public void CreatePlace(string placeName)
		{
			string placeKey = "/places/"+Guid.NewGuid().ToString();
			this.sd[placeKey] = placeName;

			// make a chat item for the new place
			string key = placeKey+"/items/"+Guid.NewGuid().ToString();

			// make a colour scheme for the place
			ColourGenerator.ColourGenerator cgen = new ColourGenerator.ColourGenerator();
			GroupLab.Networking.Map colour = new Map();
			colour["red"] = (int)cgen.BaseColour.R;
			colour["green"] = (int)cgen.BaseColour.G;
			colour["blue"] = (int)cgen.BaseColour.B;
			this.sd[placeKey+"/colour"] = colour;

			GroupLab.Networking.Map details = new Map();
			details["type"] = "Chat";
			details["owner"] = placeKey;
			details["place"] = placeKey;

			this.sd[key] = details;
		}

		public Owner Owner
		{
			get
			{
				return this.owner;
			}
		}

		public ICollection Places
		{
			get
			{
				return this.places.Values;
			}
		}

		public Place GetPlace(string key)
		{
			return (Place)this.places[key];
		}

		public Person GetPerson(string key)
		{
			return (Person)this.people[key];
		}

		public event EventHandler Started;
		protected virtual void OnStarted(EventArgs e)
		{
			if(null != this.Started)
			{
				this.Started(this, e);
			}
		}
		public event EventHandler Stopped;
		protected virtual void OnStopped(EventArgs e)
		{
			if(null != this.Stopped)
			{
				this.Stopped(this, e);
			}
		}

		public event PlaceEventHandler PlaceAdded;
		protected virtual void OnPlaceAdded(PlaceEventArgs e)
		{
			if(null != this.PlaceAdded)
			{
				this.PlaceAdded(this, e);
			}
		}
		public event PlaceEventHandler PlaceRemoved;
		protected virtual void OnPlaceRemoved(PlaceEventArgs e)
		{
			if(null != this.PlaceRemoved)
			{
				this.PlaceRemoved(this, e);
			}
		}
		public event PersonEventHandler PersonAdded;
		protected virtual void OnPersonAdded(PersonEventArgs e)
		{
			if(null != this.PersonAdded)
			{
				this.PersonAdded(this, e);
			}
		}
		public event PersonEventHandler PersonRemoved;
		protected virtual void OnPersonRemoved(PersonEventArgs e)
		{
			if(null != this.PersonRemoved)
			{
				this.PersonRemoved(this, e);
			}
		}
		public DialogResult PromptForLogin() 
		{ 
			this.loginForm = new PromptForLoginForm();
			return this.loginForm.ShowDialog(); 
		}

		public void Start()
		{
			this.deliberateStop = false;

			this.ownerName = this.loginForm.Name;
			this.ownerEmail = this.loginForm.Email;
			
			// start the shared dictionary
			if(this.sd.Url != this.loginForm.ServerName)
			{
				this.sd.Url = this.loginForm.ServerName;
			}
			this.sd.Open();
		}

		public void Start(string ownerName, string ownerEmail, string serverMachine, string serverName)
		{
			this.ownerName = ownerName;
			this.ownerEmail = ownerEmail;
			this.serverName = "tcp://"+serverMachine+":"+serverName;

			// start the shared dictionary
			if(this.sd.Url != this.serverName)
			{
				this.sd.Url = this.serverName;
			}
			this.sd.Open();
		}


		public void Stop()
		{
			this.deliberateStop = true;
			// try to close down all the items properly
			ArrayList delItems = new ArrayList();
			foreach(Place place in this.Places)
			{
				foreach(Item item in place.Items)
				{
					if(item.Owner == this.owner)
					{
						delItems.Add(item);
					}
				}
			}
			foreach(Item item in delItems)
			{
				item.Delete();
			}
			this.sd.Close();
		}

		private void sd_Opened(object sender, System.EventArgs e)
		{
			// init the object reference storage
			this.places = new Hashtable();
			this.people = new Hashtable();

			// Create/restore the owner for this connection
			this.owner = new Owner(this.sd, this, this.ownerEmail, this.ownerName);
			this.owner.Activated += new PersonEventHandler(this.owner_Activated);
			this.owner.UnsubscribedPlace += new PlaceEventHandler(this.owner_UnsubscribedPlace);

			this.owner.Activate();
		}

		public ICollection People
		{
			get
			{
				return this.people.Values;
			}
		}

		private void placesChanged_Notified(object sender, GroupLab.Networking.SubscriptionEventArgs e)
		{
			if (GroupLab.Networking.SubscriptionNotification.Add == e.Reason)
			{
				Place place = new Place(this.sd, this, e.Path, (string)e.Value);
				if(!this.places.Contains(e.Path))
				{
					this.places.Add(e.Path, place);
					PlaceEventArgs pe = new PlaceEventArgs(place);
					this.OnPlaceAdded(pe);
				}
			}
			else if (GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
			{
				Place place = (Place)this.places[e.Path];
				this.places.Remove(e.Path);
				PlaceEventArgs pe = new PlaceEventArgs(place);
				this.OnPlaceRemoved(pe);
			}
		}

		private void peopleChanged_Notified(object sender, GroupLab.Networking.SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason)
			{
				// make a person, subscribe to the activation, and activate it
				Person p = new Person(this.sd, this, e.Path);
				p.Activated += new PersonEventHandler(this.p_Activated);
				p.Activate();
			}
			else if (GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
			{
				Person p = (Person)this.people[e.Path];
				this.people.Remove(e.Path);
				this.OnPersonRemoved(new PersonEventArgs(p));
			}
		}

		private void owner_Activated(object sender, PersonEventArgs e)
		{
			this.OnStarted(EventArgs.Empty);

			// place events
			this.placesChanged = new GroupLab.Networking.Subscription();
			this.placesChanged.BeginInit();
			this.placesChanged.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.placesChanged_Notified);
			this.placesChanged.Pattern = "/places/?";
			this.placesChanged.SubscriptionChanged += new SubscriptionChangedEventHandler(this.placesChanged_SubscriptionChanged);
			this.placesChanged.Enabled = true;
			this.placesChanged.Dictionary = this.sd;
			this.placesChanged.EndInit();

			// people events
			this.peopleChanged = new GroupLab.Networking.Subscription();
			this.peopleChanged.BeginInit();
			this.peopleChanged.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.peopleChanged_Notified);
			this.peopleChanged.Pattern = "/users/?";
			this.peopleChanged.Enabled = true;
			this.peopleChanged.Dictionary = this.sd;
			this.peopleChanged.EndInit();
		}

		private void owner_UnsubscribedPlace(object sender, PlaceEventArgs e)
		{
			e.Place.DeActivate();
		}

		private void p_Activated(object sender, PersonEventArgs e)
		{
			if(!this.people.Contains(e.Person.Key))
			{
				this.people.Add(e.Person.Key, e.Person);
				this.OnPersonAdded(e);
			}
		}

		private void placesChanged_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
		}

		private void sd_Disconnected(object sender, DisconnectedEventArgs e)
		{
			this.sd.Remove("*");
			this.OnStopped(EventArgs.Empty);
			if (!this.deliberateStop)
			{
				e.RetryCount = 1;
				e.RetryTimeout = 5000;

				GroupLab.Networking.Troubleshooter.Show(this.sd, e);
//				// disconnection was not intended - query for reconnection
//				string message = "Failed connection to server: " + this.loginForm.ServerName + "\nTry again?";
//
//				if (DialogResult.Yes == MessageBox.Show(message, "Connection Query", MessageBoxButtons.YesNo))
//				{
//					e.Retry = true;
//				}
			}
		}
	}
}
