using System;
using GroupLab.Networking;
using System.Collections;

namespace CbModel
{
	/// <summary>
	/// Summary description for Owner.
	/// </summary>
	public class Owner : Person
	{
		private GroupLab.Networking.Subscription subscriptionChange;
		private bool subscriptionChangeActive;
		private GroupLab.Networking.Subscription contactListChange;
		private bool contactListChangeActive;
		private GroupLab.Networking.Subscription invitationChange;
		private bool invitationChangeActive;

		private GroupLab.Networking.Subscription defaultPresence;
		private bool defaultPresenceActive;

		private Item defaultPresenceItem;

		private ArrayList subscribedPlaceKeys;
		private Hashtable contactLists;
		private Hashtable invitations;

		public Owner(GroupLab.Networking.SharedDictionary sd, MainModel mainModel, string email, string displayName) : base(sd,mainModel,email,displayName)
		{
			//this.mainModel = mainModel;
		}

		public Owner(GroupLab.Networking.SharedDictionary sd, MainModel mainModel, string key) : base(sd, mainModel, key)
		{
			//this.mainModel = mainModel;
		}

		protected override void InitPerson()
		{
			base.InitPerson ();
			this.subscribedPlaceKeys = new ArrayList();
			this.contactLists = new Hashtable();
			this.invitations = new Hashtable();

			this.subscriptionChangeActive = false;
			this.contactListChangeActive = false;
			this.defaultPresenceActive = false;

			this.sd[this.Key+"/online"] = "online";
			this.sd[this.Key+"/online.transient"] = this.sd.Id;

		}

		public bool IsInvited(Place place)
		{
			return (this.invitations.Contains(place.Key));
		}

		public override bool Equals(object obj)
		{
			return this.GetHashCode() == obj.GetHashCode();
		}

		public override int GetHashCode()
		{
			return this.Key.GetHashCode();
		}

		public override void Activate()
		{
			base.Activate();
			this.subscriptionChange = new Subscription();
			this.subscriptionChange.BeginInit();
			this.subscriptionChange.Notified += new SubscriptionEventHandler(subscriptionChange_Notified);
			this.subscriptionChange.SubscriptionChanged += new SubscriptionChangedEventHandler(this.subscriptionChange_SubscriptionChanged);
			this.subscriptionChange.Pattern = "/places/?/member/?";
			this.subscriptionChange.Enabled = true;
			this.subscriptionChange.Dictionary = this.sd;
			this.subscriptionChange.EndInit();

			if(this.mainModel.ContactsMode)
			{
				this.contactListChange = new Subscription();
				this.contactListChange.BeginInit();
				this.contactListChange.Notified += new SubscriptionEventHandler(this.contactListChange_Notified);
				this.contactListChange.SubscriptionChanged += new SubscriptionChangedEventHandler(this.contactListChange_SubscriptionChanged);
				this.contactListChange.Pattern = this.Key+"/contactlist/?/?";
				this.contactListChange.Enabled = true;
				this.contactListChange.Dictionary = this.sd;
				this.contactListChange.EndInit();
			}

			this.invitationChange = new Subscription();
			this.invitationChange.BeginInit();
			this.invitationChange.Notified += new SubscriptionEventHandler(this.invitationChange_Notified);
			this.invitationChange.SubscriptionChanged += new SubscriptionChangedEventHandler(this.invitationChange_SubscriptionChanged);
			this.invitationChange.Pattern = "/places/?/invited/?";
			this.invitationChange.Enabled = true;
			this.invitationChange.Dictionary = this.sd;
			this.invitationChange.EndInit();

			if(this.mainModel.ContactsMode)
			{
				this.defaultPresence = new Subscription();
				this.defaultPresence.BeginInit();
				this.defaultPresence.SubscriptionChanged += new SubscriptionChangedEventHandler(this.defaultPresence_SubscriptionChanged);
				this.defaultPresence.Notified += new SubscriptionEventHandler(this.defaultPresence_Notified);
				this.defaultPresence.Pattern = this.Key+"/default_presence";
				this.defaultPresence.Enabled = true;
				this.defaultPresence.Dictionary = this.sd;
				this.defaultPresence.EndInit();
			}
		}

		public override void DeActivate()
		{
			base.DeActivate();
			this.subscriptionChangeActive = false;
			this.subscriptionChange.Enabled = false;
		}
		public void SubscribePlace(Place place)
		{
			// doesn't assume the place is active as it most likely won't be before we subscribe
			string memberKey = place.Key + "/member/"+Guid.NewGuid().ToString();
			this.sd[memberKey] = this.Key;
			this.sd[memberKey+"/focus"] = 100;
		}

		public void UnsubscribePlace(Place place)
		{
			// assumes the place is active
			place.Unsubscribe(this);
		}

		public Item Invitation(Place place)
		{
			return (Item)this.invitations[place];
		}

		public event PlaceEventHandler SubscribedPlace;
		protected virtual void OnSubscribedPlace(PlaceEventArgs e)
		{
			if(null != this.SubscribedPlace)
			{
				this.SubscribedPlace(this, e);
			}
		}
		public event PlaceEventHandler UnsubscribedPlace;
		protected virtual void OnUnsubscribedPlace(PlaceEventArgs e)
		{
			if(null != this.UnsubscribedPlace)
			{
				this.UnsubscribedPlace(this, e);
			}
		}
		public event PlaceEventHandler InvitedPlace;
		protected virtual void OnInvitedPlace(PlaceEventArgs e)
		{
			if(null != this.InvitedPlace)
			{
				this.InvitedPlace(this, e);
			}
		}
		public event PlaceEventHandler UninvitedPlace;
		protected virtual void OnUninvitedPlace(PlaceEventArgs e)
		{
			if(null != this.UninvitedPlace)
			{
				this.UninvitedPlace(this, e);
			}
		}
		public event ContactListEventHandler ContactListAdded;
		protected virtual void OnContactListAdded(ContactListEventArgs e)
		{
			if(null != this.ContactListAdded)
			{
				this.ContactListAdded(this, e);
			}
		}
		public event ContactListEventHandler ContactListRemoved;
		protected virtual void OnContactListRemoved(ContactListEventArgs e)
		{
			if(null != this.ContactListRemoved)
			{
				this.ContactListRemoved(this, e);
			}
		}

		protected override void CheckActive()
		{
			if(this.subscriptionChangeActive && this.nameChangeActive && this.invitationChangeActive && ((this.ownerContactActive && this.contactListChangeActive  && this.defaultPresenceActive) || !this.mainModel.ContactsMode))
			{
				this.OnActivated(new PersonEventArgs(this));
			}
		}

		private void subscriptionChange_Notified(object sender, SubscriptionEventArgs e)
		{
			
			if((e.Value == null && (string)e.PreviousValue == this.Key) || (string)e.Value == this.Key)
			{
				// reconstruct the place
				int slashIndex = e.Path.LastIndexOf("/member");
				string groupKey = e.Path.Substring(0,slashIndex);
				Place place = new Place(this.sd, this.mainModel, groupKey);

				if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason)
				{
					place.Activated += new PlaceEventHandler(this.subscribedPlace_Activated);
					place.Activate();
				}
				else if (GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
				{
					this.subscribedPlaceKeys.Remove(place.Key);
					PlaceEventArgs pe = new PlaceEventArgs(place);
					this.OnUnsubscribedPlace(pe);
				}
			}
		}

		private void subscribedPlace_Activated(object sender, PlaceEventArgs e)
		{
			this.subscribedPlaceKeys.Add(e.Place.Key);
			this.OnSubscribedPlace(e);
		}

		private void invitedPlace_Activated(object sender, PlaceEventArgs e)
		{
			// just been invited
			this.OnInvitedPlace(e);
		}

		private void subscriptionChange_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.subscriptionChangeActive)
			{
				this.subscriptionChangeActive = true;
				this.CheckActive();
			}
		}

		public bool IsSubscribed(Place place)
		{
			return this.subscribedPlaceKeys.Contains(place.Key);
		}

		// Contact List stuff

		public void CreateContactList(string name)
		{
			GroupLab.Networking.Map details = new GroupLab.Networking.Map();
			details["owner"] = this.Key;
			details["type"] = "Presence";
			details["name"] = name;
			this.sd[this.Key+"/contactlist/"+Guid.NewGuid().ToString()+"/presence"] = details;
		}

		private void contactListChange_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason)
			{
				// make a new contact list object
				ContactList contacts = new ContactList(this.sd, this.mainModel, this, e.Path);
				// add it to the contact list dictionary
				this.contactLists.Add(e.Path, contacts);
				contacts.Activated += new ContactListEventHandler(this.contacts_Activated);
				contacts.Activate();
			}
			else if(GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
			{
				// get the contact list
				ContactList list = (CbModel.ContactList)this.contactLists[e.Path];
				// remove the contact list
				this.contactLists.Remove(e.Path);
				// tell everyone
				ContactListEventArgs cle = new ContactListEventArgs(list);
				this.OnContactListRemoved(cle);
			}

		}

		private void contactListChange_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.contactListChangeActive)
			{
				this.contactListChangeActive = true;
				this.CheckActive();
			}
		}

		private void contacts_Activated(object sender, ContactListEventArgs e)
		{
			// tell everyone about it
			ContactListEventArgs cle = new ContactListEventArgs(e.ContactList);
			this.OnContactListAdded(cle);
		}

		private void invitationChange_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.invitationChangeActive)
			{
				this.invitationChangeActive = true;
				this.CheckActive();
			}
		}

		private void invitationChange_Notified(object sender, SubscriptionEventArgs e)
		{
			// check that it's us that is being invited
			GroupLab.Networking.Map info = null;
			if(e.Value == null)
			{
				info = e.PreviousValue as GroupLab.Networking.Map;
			}
			else
			{
				info = e.Value as GroupLab.Networking.Map;
			}
			if(info != null && info.Contains("invitee") && (info["invitee"] as string) == this.Key)
			{
				// reconstruct the place
				int slashIndex = e.Path.LastIndexOf("/invited");
				string groupKey = e.Path.Substring(0,slashIndex);
				Place place = new Place(this.sd, this.mainModel, groupKey);

				if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason)
				{
					// this is early - make a difference?
					Item item = new Item(this.sd, this.mainModel, e.Path);
					this.invitations.Add(place.Key, item);

					place.Activated += new PlaceEventHandler(this.invitedPlace_Activated);
					place.Activate();
				}
				else if (GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
				{
					this.invitations.Remove(place.Key);
					PlaceEventArgs pe = new PlaceEventArgs(place);
					this.OnUninvitedPlace(pe);
				}
			}

		}

		private void defaultPresence_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			// create the default presence item
			GroupLab.Networking.Map details = new Map();
			details["type"] = "Presence";
			details["owner"] = this.Key;
			details["place"] = string.Empty;
			this.sd[this.Key+"/default_presence"] = details;
			this.sd[this.Key+"/default_presence.transient"] = this.sd.Id;
		}

		private void defaultPresence_Notified(object sender, SubscriptionEventArgs e)
		{
			this.defaultPresenceItem = new Item(this.sd, this.mainModel, e.Path);
			if(!this.defaultPresenceActive)
			{
				this.defaultPresenceActive = true;
				this.CheckActive();
			}
		}
	}
}
