using System;
using System.Collections;

namespace CbModel
{
	/// <summary>
	/// Summary description for Person.
	/// </summary>
	public class Person
	{
		protected GroupLab.Networking.SharedDictionary sd;
		protected CbModel.MainModel mainModel;

		// base path for this person's info in the shared dictionary
		string key;
		string email;

		// the presence item that this person shows to the owner
		CbModel.Item presenceItem;

		GroupLab.Networking.Subscription nameChange;
		GroupLab.Networking.Subscription statusChange;
		GroupLab.Networking.Subscription ownerContact;
		GroupLab.Networking.Subscription presence;

		protected bool nameChangeActive;
		protected bool statusChangeActive;
		protected bool ownerContactActive;
		protected bool presenceActive;

		public Person(GroupLab.Networking.SharedDictionary sd, CbModel.MainModel mainModel, string email, string displayName)
		{
			key = "/users/`"+email+"`";
			sd[key] = displayName;

			this.sd = sd;
			this.mainModel = mainModel;
			this.key = key;
			this.email = email;

			this.InitPerson();
		}

		public Person(GroupLab.Networking.SharedDictionary sd, CbModel.MainModel mainModel, string key)
		{
			this.sd = sd;
			this.mainModel = mainModel;
			this.key = key;

			// extract the email from the key
			this.email = this.key.Substring(8,this.key.Length-9);

			this.InitPerson();
		}

		// put all the initialisation here because I can't work out how to call one 
		// constructor from the other
		protected virtual void InitPerson()
		{
			this.nameChangeActive = false;
			this.statusChangeActive = false;
			this.ownerContactActive = false;
			this.presenceActive = false;
		}
		public static bool operator != (Person a, Person b)
		{
			if(object.ReferenceEquals(a, null))
			{
				if(object.ReferenceEquals(b, null))
				{
					return false;
				}
				else
				{
					return true;
				}
			}
			else if(object.ReferenceEquals(b, null))
			{
				return true;
			}
			else
			{
				return (0 != string.Compare(a.Key, b.Key, true));
			}
		}		
		public static bool operator == (Person a, Person b)
		{
			if(object.ReferenceEquals(a, null))
			{
				if(object.ReferenceEquals(b, null))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else if(object.ReferenceEquals(b, null))
			{
				return false;
			}
			else
			{
				return (0 == string.Compare(a.Key, b.Key, true));
			}
		}
		public override bool Equals(object obj)
		{
			return (!object.ReferenceEquals(null, obj) && (this.GetHashCode() == obj.GetHashCode()));
		}

		public override int GetHashCode()
		{
			return this.Key.GetHashCode();
		}

		public virtual void Activate()
		{
			this.nameChange = new GroupLab.Networking.Subscription();
			this.nameChange.BeginInit();
			this.nameChange.SubscriptionChanged += new GroupLab.Networking.SubscriptionChangedEventHandler(this.nameChange_SubscriptionChanged);
			this.nameChange.Pattern = this.key;
			this.nameChange.Enabled = true;
			this.nameChange.Dictionary = this.sd;
			this.nameChange.EndInit();

			this.statusChange = new GroupLab.Networking.Subscription();
			this.statusChange.BeginInit();
			this.statusChange.SubscriptionChanged += new GroupLab.Networking.SubscriptionChangedEventHandler(this.statusChange_SubscriptionChanged);
			this.statusChange.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.statusChange_Notified);
			this.statusChange.Pattern = this.key+"/online";
			this.statusChange.Enabled = true;
			this.statusChange.Dictionary = this.sd;
			this.statusChange.EndInit();

			if(this.mainModel.ContactsMode)
			{
				this.ownerContact = new GroupLab.Networking.Subscription();
				this.ownerContact.BeginInit();
				this.ownerContact.SubscriptionChanged += new GroupLab.Networking.SubscriptionChangedEventHandler(this.ownerContact_SubscriptionChanged);
				this.ownerContact.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.ownerContact_Notified);
				this.ownerContact.Pattern = this.key+this.mainModel.Owner.Key;
				this.ownerContact.Enabled = true;
				this.ownerContact.Dictionary = this.sd;
				this.ownerContact.EndInit();

				this.InitPresence();
			}
		}

		public virtual void DeActivate()
		{
			this.nameChangeActive = false;
			this.nameChange.Enabled = false;
		}

		public virtual bool Active
		{
			get 
			{
				return this.nameChangeActive && this.statusChangeActive;
			}
		}
		public string DisplayName
		{
			get
			{
				if (this.Active)
				{
					if(null != this.sd && this.sd.Contains(this.Key))
					{
						return (string)this.sd[this.key];
					}
					else
					{
						return string.Empty;
					}
				}
				else
				{
					throw new Exception("Person not active yet.");
				}
			}
		}

		public string Key
		{
			get 
			{
				return this.key;
			}
		}

		public override string ToString()
		{
			return this.DisplayName;
		}

		public string Status
		{
			get
			{
				if(this.sd.Contains(this.key+"/online"))
				{
					return "Online";
				}
				else
				{
					return "Offline";
				}
			}
		}


		// Events
		public event EventHandler NameChanged;
		protected virtual void OnNameChanged(EventArgs e)
		{
			if(null != this.NameChanged)
			{
				this.NameChanged(this, e);
			}
		}
		public event PersonEventHandler Activated;
		protected virtual void OnActivated(PersonEventArgs e)
		{
			if(null != this.Activated)
			{
				this.Activated(this, e);
			}
		}
		public event PersonEventHandler StatusChanged;
		protected virtual void OnStatusChanged(PersonEventArgs e)
		{
			if(null != this.StatusChanged)
			{
				this.StatusChanged(this, e);
			}
		}
		public event PersonEventHandler PresenceChanged;
		protected virtual void OnPresenceChanged(PersonEventArgs e)
		{
			if(null != this.PresenceChanged)
			{
				this.PresenceChanged(this, e);
			}
		}

		private void nameChange_Notified(object sender, GroupLab.Networking.SubscriptionEventArgs e)
		{
			
			EventArgs ea = new EventArgs();
			this.OnNameChanged(ea);
		}

		protected virtual void CheckActive()
		{
			bool yesgo = false;
			if(this.nameChangeActive)
			{
				if(this.statusChangeActive)
				{
					if(this.presenceActive || !this.mainModel.ContactsMode)
					{
						yesgo = true;
					}
				}
			}
			if(yesgo)
			{
				this.OnActivated(new PersonEventArgs(this));
			}
		}

		private void nameChange_SubscriptionChanged(object sender, GroupLab.Networking.SubscriptionChangedEventArgs e)
		{
			if(!this.nameChangeActive)
			{
				this.nameChangeActive = true;
				this.CheckActive();
			}
		}

		private void statusChange_SubscriptionChanged(object sender, GroupLab.Networking.SubscriptionChangedEventArgs e)
		{
			if (!this.statusChangeActive)
			{
				this.statusChangeActive = true;
				this.CheckActive();
			}
		}

		private void statusChange_Notified(object sender, GroupLab.Networking.SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason ||
				GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
			{
				PersonEventArgs pe = new PersonEventArgs(this);
				this.OnStatusChanged(pe);
			}
		}

		private void ownerContact_SubscriptionChanged(object sender, GroupLab.Networking.SubscriptionChangedEventArgs e)
		{
			if(!this.ownerContactActive)
			{
				this.ownerContactActive = true;
//				if(this.sd.Contains(this.key+this.mainModel.Owner.Key))
//				{
//					// the owner is listed in the contacts - set up the presence sub
//					this.InitPresence((string)this.sd[this.key+this.mainModel.Owner.Key]);
//				}
			}
		}

		private void ownerContact_Notified(object sender, GroupLab.Networking.SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason || GroupLab.Networking.SubscriptionNotification.Change == e.Reason)
			{
				this.presenceActive = false;
				this.presence.Pattern = (string)e.Value;
			}
			else if(GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
			{
				this.presenceActive = false;
				this.presence.Pattern = this.key+"/default_presence";
			}
		}

		private void InitPresence()
		{
			this.presence = new GroupLab.Networking.Subscription();
			this.presence.BeginInit();
			this.presence.Pattern = this.key+"/default_presence";
			this.presence.Dictionary = this.sd;
			this.presence.SubscriptionChanged += new GroupLab.Networking.SubscriptionChangedEventHandler(this.presence_SubscriptionChanged);
			this.presence.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.presence_Notified);
			this.presence.Enabled = true;
			this.presence.EndInit();
		}

		private void presence_SubscriptionChanged(object sender, GroupLab.Networking.SubscriptionChangedEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason ||
				GroupLab.Networking.SubscriptionNotification.Change == e.Reason)
			{
				this.presenceActive = true;
				this.CheckActive();
			}
		}

		private void presence_Notified(object sender, GroupLab.Networking.SubscriptionEventArgs e)
		{
			PersonEventArgs pe = new PersonEventArgs(this);
			this.OnPresenceChanged(pe);
		}

		public CbModel.Item GetPresenceForOwner()
		{
			if(this.presenceActive)
			{
				this.presenceItem = new Item(this.sd, this.mainModel, this.presence.Pattern);
			}
			else
			{
				// owner is not listed - return the default presence
				this.presenceItem = new Item(this.sd, this.mainModel, this.key+"/default_presence");
			}
			return this.presenceItem;
		}
	}
}
