using System;
using System.Collections;
using GroupLab.Networking;
//using Collabrary;

namespace CbModel
{
	/// <summary>
	/// Summary description for Place.
	/// </summary>
		
	public class Place
	{
		const string DefaultName = "Default Place Name";
		GroupLab.Networking.SharedDictionary sd;

		string key;

		MainModel mainModel;

		object tag;

		private GroupLab.Networking.Subscription itemChange;
		private GroupLab.Networking.Subscription nameChange;
		private GroupLab.Networking.Subscription personChange;
		private GroupLab.Networking.Subscription colourSub;
		private GroupLab.Networking.Subscription ownerInfoSub;

		// [Nata] for logging purpose
		private GroupLab.Networking.Subscription toolTipDisplay;
		private GroupLab.Networking.Subscription separateDisplay;

		// here because we want it to persist between sessions
		private GroupLab.Networking.Subscription focusChange;

		private Hashtable items;
		private Hashtable people;

		// store the name for when the place is not active
		private string name;

		private bool itemChangeActive;
		private bool nameChangeActive;
		private bool personChangeActive;
		private bool focusChangeActive;
		private bool colourActive;
		private bool ownerInfoActive;

		// [Nata] for logging purpose
		private bool toolTipDisplayActive;
		private bool separateDisplayActive;


		private ColourGenerator.ColourGenerator cgen;

		public Place(GroupLab.Networking.SharedDictionary sd, MainModel mainModel, string key, string name)
		{
			this.sd = sd;
			this.mainModel = mainModel;
			this.key = key;
			this.name = name;

			this.items = new Hashtable();
			this.people = new Hashtable();
			//this.InitPeople();

			this.itemChangeActive = false;
			this.nameChangeActive = false;
			this.personChangeActive = false;
			this.focusChangeActive = false;
			this.colourActive = false;
			this.ownerInfoActive = false;

			// [Nata] for logging purpose
			this.toolTipDisplayActive = false;
			this.separateDisplayActive = false;
		}

		public Place(GroupLab.Networking.SharedDictionary sd, MainModel mainModel, string key) : this(sd, mainModel, key, DefaultName)
		{ }

		public void Activate()
		{
			// create the subscriptions
			this.nameChange = new GroupLab.Networking.Subscription();
			this.nameChange.BeginInit();
			this.nameChange.SubscriptionChanged += new SubscriptionChangedEventHandler(this.nameChange_SubscriptionChanged);
			this.nameChange.Notified += new SubscriptionEventHandler(this.nameChange_Notified);
			this.nameChange.Pattern = this.Key;
			this.nameChange.Enabled = true;
			this.nameChange.Dictionary = this.sd;
			this.nameChange.EndInit();

			this.itemChange = new GroupLab.Networking.Subscription();
			this.itemChange.BeginInit();
			this.itemChange.SubscriptionChanged += new SubscriptionChangedEventHandler(this.itemChange_SubscriptionChanged);
			this.itemChange.Notified += new SubscriptionEventHandler(this.itemsChanged_Notified);
			this.itemChange.Pattern = this.Key + "/items/?";
			this.itemChange.Enabled = true;
			this.itemChange.Dictionary = this.sd;
			this.itemChange.EndInit();

			this.personChange = new GroupLab.Networking.Subscription();
			this.personChange.BeginInit();
			this.personChange.SubscriptionChanged += new SubscriptionChangedEventHandler(this.personChange_SubscriptionChanged);
			this.personChange.Notified += new SubscriptionEventHandler(this.personChange_Notified);
			this.personChange.Pattern = this.Key + "/member/?";
			this.personChange.Enabled = true;
			this.personChange.Dictionary = this.sd;
			this.personChange.EndInit();

			this.focusChange = new GroupLab.Networking.Subscription();
			this.focusChange.BeginInit();
			this.focusChange.Pattern = this.Key+this.mainModel.Owner.Key+"/focus";
			this.focusChange.Enabled = true;
			this.focusChange.Dictionary = this.sd;
			this.focusChange.SubscriptionChanged += new SubscriptionChangedEventHandler(this.focusChange_SubscriptionChanged);
			this.focusChange.Notified += new SubscriptionEventHandler(this.focusChange_Notified);
			this.focusChange.EndInit();

			this.colourSub = new GroupLab.Networking.Subscription();
			this.colourSub.BeginInit();
			this.colourSub.Pattern = this.Key+"/colour";
			this.colourSub.Enabled = true;
			this.colourSub.Dictionary = this.sd;
			this.colourSub.SubscriptionChanged += new SubscriptionChangedEventHandler(this.colourSub_SubscriptionChanged);
			this.colourSub.EndInit();

			this.ownerInfoSub = new GroupLab.Networking.Subscription();
			this.ownerInfoSub.BeginInit();
			this.ownerInfoSub.Pattern = this.Key+this.mainModel.Owner.Key+"/*";
			this.ownerInfoSub.Enabled = true;
			this.ownerInfoSub.Dictionary = this.sd;
			this.ownerInfoSub.SubscriptionChanged += new SubscriptionChangedEventHandler(this.ownerInfoSub_SubscriptionChanged);
			this.ownerInfoSub.EndInit();

			// [Nata]: for logging purpose
			this.toolTipDisplay = new GroupLab.Networking.Subscription();
			this.toolTipDisplay.BeginInit();
			this.toolTipDisplay.Pattern = this.Key+this.mainModel.Owner.Key+"/tooltip_place";
			this.toolTipDisplay.Enabled = true;
			this.toolTipDisplay.Dictionary = this.sd;
			this.toolTipDisplay.SubscriptionChanged += new SubscriptionChangedEventHandler(this.toolTipDisplay_SubscriptionChanged);
			this.toolTipDisplay.EndInit();

			// [Nata]: for logging purpose
			this.separateDisplay = new GroupLab.Networking.Subscription();
			this.separateDisplay.BeginInit();
			this.separateDisplay.Pattern = this.Key+this.mainModel.Owner.Key+"/separate_place";
			this.separateDisplay.Enabled = true;
			this.separateDisplay.Dictionary = this.sd;
			this.separateDisplay.SubscriptionChanged += new SubscriptionChangedEventHandler(this.separateDisplay_SubscriptionChanged);
			this.separateDisplay.EndInit();
		}

		// [Nata] for logging purpose
		public void logToolTipPlaceDisplay()
		{
			this.sd[this.toolTipDisplay.Pattern] = "display";
		}

		// [Nata] for logging purpose
		public void logToolTipPlaceDispose()
		{
			this.sd[this.toolTipDisplay.Pattern] = "dispose";
		}

		// [Nata] for logging purpose
		public void logPlaceSeparateDisplay()
		{
			this.sd[this.separateDisplay.Pattern] = "display";
		}

		// [Nata] for logging purpose
		public void logPlaceSeparateClose()
		{
			this.sd[this.separateDisplay.Pattern] = "close";
		}

		public void DeActivate()
		{
			if(this.Active)
			{
				this.nameChangeActive = false;
				this.itemChangeActive = false;
				this.personChangeActive = false;
				this.focusChangeActive = false;
				this.colourActive = false;
				this.ownerInfoActive = false;

				// [Nata] for logging purpose
				this.toolTipDisplayActive = false;

				this.nameChange.Enabled = false;
				this.itemChange.Enabled = false;
				this.personChange.Enabled = false;
				this.focusChange.Enabled = false;
				this.ownerInfoSub.Enabled = false;

				// [Nata] for logging purpose
				this.toolTipDisplay.Enabled = false;
			}
		}

		public ColourGenerator.ColourGenerator Colour
		{
			get
			{
				if(null == this.cgen)
				{
					// get the starter colour
					GroupLab.Networking.Map colour = (this.sd[this.Key+"/colour"] as GroupLab.Networking.Map);
					if(null != colour)
					{
						System.Drawing.Color baseColour = System.Drawing.Color.FromArgb((int)colour["red"], (int)colour["green"], (int)colour["blue"]);
						this.cgen = new ColourGenerator.ColourGenerator(baseColour);
					}
				}
				return this.cgen;
			}
		}

		public int Focus
		{
			get
			{
				if(null != this.sd && this.sd.Contains(this.Key+this.mainModel.Owner.Key+"/focus"))
				{
					return (int)this.sd[this.Key+this.mainModel.Owner.Key+"/focus"];
				}
				else
				{
					return 100;
				}
			}
			set
			{
				this.sd[this.Key+this.mainModel.Owner.Key+"/focus"] = value;
			}
		}

		private void InitPeople()
		{
			ArrayList peopleKeys = new ArrayList();
			lock(this.sd.SyncRoot)
			{
				foreach(GroupLab.Networking.SharedDictionary.Entry entry in (this.sd[this.Key+"/member/?"] as IEnumerable))
				{
					peopleKeys.Add(entry.Value);
				}
			}
			foreach(string key in peopleKeys)
			{
				Person p = new Person(this.sd, this.mainModel, key);
				p.Activated += new PersonEventHandler(this.p_Activated);
				p.Activate();
			}
		}

		public void Delete()
		{
			this.sd.Remove(this.key+"*");
		}

		private void p_Activated(object sender, PersonEventArgs e)
		{
			this.people.Add(e.Person.Key, e.Person);
		}

		public override string ToString()
		{
			return this.Name;
		}

		public MainModel MainModel
		{
			get
			{
				return this.mainModel;
			}
		}

		public void CreateItem(string itemType)
		{
			string key = this.Key+"/items/"+Guid.NewGuid().ToString();

			GroupLab.Networking.Map details = new Map();
			details["type"] = itemType;
			details["owner"] = this.mainModel.Owner.Key;
			details["place"] = this.Key;

			this.sd[key] = details;
			this.sd[key+".transient"] = this.sd.Id;
		}

		public bool Active
		{
			get 
			{
				return (this.itemChangeActive && 
					this.nameChangeActive && 
					this.personChangeActive && 
					this.focusChangeActive && 
					this.colourActive &&
					this.ownerInfoActive);
			}
		}

		public string Name
		{
			get
			{
				if(this.Active)
				{
					return (string)this.sd[this.key];
				} 
				else
				{
					return this.name;
				}
			}
		}

		private string getKeyForThisUser()
		{
			string key = string.Empty;
			foreach(GroupLab.Networking.SharedDictionary.Entry i in (this.sd[this.key+"/member/?"] as IEnumerable))
			{
				if((string)i.Value == this.mainModel.Owner.Key)
				{
					key = i.Path;
					break;
				}
			}
			return key;
		}
		public object Tag
		{
			get
			{
				return this.tag;
			}
			set
			{
				this.tag = value;
			}
		}
		public string Key
		{
			get
			{
				return this.key;
			}
		}

		public ICollection Items
		{
			get
			{
				return this.items.Values;
			}
		}

		public int NumPeople
		{
			get
			{
				int i = 0;
				foreach(Object o in (this.sd[this.key+"/member/?"] as IEnumerable))
				{
					i++;
				}
				return i;
			}
		}
		public ICollection People
		{
			get
			{
				return this.people.Values;
			}
		}

		public void Subscribe(Person person)
		{
			if(!this.IsSubscribed(person))
			{
				string memberKey = this.key+"/member/"+Guid.NewGuid().ToString();
				this.sd[memberKey] = person.Key;
				this.sd[memberKey+"/focus"] = 100;
			}
		}

		public void Unsubscribe(Person person)
		{
			ArrayList removalKeys = new ArrayList();
			foreach(GroupLab.Networking.SharedDictionary.Entry i in (this.sd[this.key+"/member/?"] as IEnumerable))
			{
				if((string)i.Value == person.Key)
				{
					removalKeys.Add(i.Path);
				}
			}
			foreach(string path in removalKeys)
			{
				this.sd.Remove(path);
				if(this.sd.Contains(path+"/focus"))
				{
					this.sd.Remove(path+"/focus");
				}
			}
			if(this.sd.Contains(this.Key+"/"+person.Key+"/*"))
			{
				this.sd.Remove(this.Key+"/"+person.Key+"/*");
			}
		}

		public override bool Equals(object obj)
		{
			return this.GetHashCode() == obj.GetHashCode();
		}

		public override int GetHashCode()
		{
			return this.Key.GetHashCode();
		}

		public bool IsSubscribed(Person person)
		{
			return this.people.Contains(person.Key);
		}

		private void itemsChanged_Notified(object sender, GroupLab.Networking.SubscriptionEventArgs e)
		{			
			if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason)
			{
				if(this.sd.Contains(e.Path))
				{
					Item item = new Item(this.sd, this.mainModel, e.Path);
					this.items.Add(e.Path, item);
					ItemEventArgs ie = new ItemEventArgs(item);
					this.OnItemAdded(ie);
				}
			}
			else if (GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
			{
				Item item = (Item)this.items[e.Path];
				this.items.Remove(e.Path);

				ItemEventArgs ie = new ItemEventArgs(item);
				this.OnItemRemoved(ie);
			}
		}

		// Event stuff
		public event ItemEventHandler ItemAdded;
		protected virtual void OnItemAdded(ItemEventArgs e)
		{
			if(null != this.ItemAdded)
			{
				this.ItemAdded(this, e);
			}
		}
		public event ItemEventHandler ItemRemoved;
		protected virtual void OnItemRemoved(ItemEventArgs e)
		{
			if(null != this.ItemRemoved)
			{
				this.ItemRemoved(this, e);
			}
		}
		public event PersonEventHandler PersonSubscribed;
		protected virtual void OnPersonSubscribed(PersonEventArgs e)
		{
			if(null != this.PersonSubscribed)
			{
				this.PersonSubscribed(this, e);
			}
		}
		public event PersonEventHandler PersonUnsubscribed;
		protected virtual void OnPersonUnsubscribed(PersonEventArgs e)
		{
			if(null != this.PersonUnsubscribed)
			{
				this.PersonUnsubscribed(this, e);
			}
		}
		public event PlaceEventHandler Activated;
		protected virtual void OnActivate(PlaceEventArgs e)
		{
			if(null != this.Activated)
			{
				this.Activated(this, e);
			}
		}
		public event EventHandler FocusChanged;
		protected virtual void OnFocusChanged(EventArgs e)
		{
			if(null != this.FocusChanged)
			{
				this.FocusChanged(this, e);
			}
		}

		private void personChange_Notified(object sender, SubscriptionEventArgs e)
		{			
			if(GroupLab.Networking.SubscriptionNotification.Add == e.Reason)
			{
				Person person = new Person(this.sd, this.mainModel, (string)e.Value);
				if(!person.Active)
				{
					person.Activated += new PersonEventHandler(this.person_Activated);
					person.Activate();
					//this.mainModel.GetPerson((string)e.Value);
				}
				else
				{
					// person already active
					PersonEventArgs pe = new PersonEventArgs(person);
					this.person_Activated(null, pe);
				}
			}
			else if (GroupLab.Networking.SubscriptionNotification.Remove == e.Reason)
			{
				if(this.people.Contains((string)e.PreviousValue))
				{
					Person person = (CbModel.Person)this.people[(string)e.PreviousValue];
					if(person == this.mainModel.Owner)
					{
						// remove all the items that belong to this person
						foreach(Item item in this.Items)
						{
							if(person == item.Owner)
							{
								item.Delete();
							}
						}
					}
					this.people.Remove((string)e.PreviousValue);

					PersonEventArgs pe = new PersonEventArgs(person);
					this.OnPersonUnsubscribed(pe);
				}
			}
		}

		private void nameChange_Notified(object sender, SubscriptionEventArgs e)
		{
				
		}

		private void nameChange_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.nameChangeActive)
			{
				this.nameChangeActive = true;
				if(this.Active)
				{
					this.OnActivate(new PlaceEventArgs(this));
				}
			}
		}

		private void itemChange_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.itemChangeActive)
			{
				this.itemChangeActive = true;
				if(this.Active)
				{
					this.OnActivate(new PlaceEventArgs(this));
				}
			}
		}

		private void personChange_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.personChangeActive)
			{
				this.personChangeActive = true;
				if(this.Active)
				{
					this.OnActivate(new PlaceEventArgs(this));
				}
			}
		}

		private void person_Activated(object sender, PersonEventArgs e)
		{
			if(!this.IsSubscribed(e.Person))
			{
				this.people.Add(e.Person.Key, e.Person);
				this.OnPersonSubscribed(e);
			}
		}

		public static bool operator != (Place a, Place b)
		{
			if(object.ReferenceEquals(a, null))
			{
				if(object.ReferenceEquals(b, null))
				{
					return false;
				}
				else
				{
					return true;
				}
			}
			else if(object.ReferenceEquals(b, null))
			{
				return true;
			}
			else
			{
				return (0 != string.Compare(a.Key, b.Key, true));
			}
		}		
		public static bool operator == (Place a, Place b)
		{
			if(object.ReferenceEquals(a, null))
			{
				if(object.ReferenceEquals(b, null))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else if(object.ReferenceEquals(b, null))
			{
				return false;
			}
			else
			{
				return (0 == string.Compare(a.Key, b.Key, true));
			}
		}

		private void focusChange_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.focusChangeActive)
			{
				this.focusChangeActive = true;
				if(this.Active)
				{
					PlaceEventArgs pe = new PlaceEventArgs(this);
					this.OnActivate(pe);
				}
			}
		}

		private void focusChange_Notified(object sender, SubscriptionEventArgs e)
		{
			this.OnFocusChanged(new EventArgs());
		}

		//  [Nata]: For logging purposes
		private void toolTipDisplay_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.toolTipDisplayActive)
			{
				this.toolTipDisplayActive = true;
				if(this.Active)
				{
					PlaceEventArgs pe = new PlaceEventArgs(this);
					this.OnActivate(pe);
				}
			}
		}

		//  [Nata]: For logging purposes
		private void separateDisplay_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.separateDisplayActive)
			{
				this.separateDisplayActive = true;
				if(this.Active)
				{
					PlaceEventArgs pe = new PlaceEventArgs(this);
					this.OnActivate(pe);
				}
			}
		}


		private void colourSub_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.colourActive)
			{
				this.colourActive = true;
				if(this.Active)
				{
					PlaceEventArgs pe = new PlaceEventArgs(this);
					this.OnActivate(pe);
				}
			}
		}

		private void ownerInfoSub_SubscriptionChanged(object sender, SubscriptionChangedEventArgs e)
		{
			if(!this.ownerInfoActive)
			{
				this.ownerInfoActive = true;
				if(this.Active && this.sd.Contains(this.Key))
				{
					PlaceEventArgs pe = new PlaceEventArgs(this);
					this.OnActivate(pe);
				}
			}					
		}
	}
}

