using System;
using System.Drawing;
using System.Collections;

namespace ColourGenerator
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class ColourGenerator
	{
		private System.Drawing.Color baseColour;

		public ColourGenerator()
		{
			this.CreateBaseColour();

			//this.TestHSLtoRGB();
			//this.TestRGBtoHSL();
		}

		public ColourGenerator(System.Drawing.Color baseColour)
		{
			this.baseColour = baseColour;
		}

		public System.Drawing.Color BaseColour
		{
			get
			{
				return this.baseColour;
			}
			set
			{
				this.baseColour = value;
			}
		}

		public void ResetColour()
		{
			this.CreateBaseColour();
		}

		public System.Drawing.Color NewColour
		{
			get
			{
				// increase the saturation of the base colour
				Hashtable hsl = this.RGBtoHSL(this.baseColour.R, this.baseColour.G, this.baseColour.B);
				float lightness = 0.5F;
				Hashtable rgb = this.HSLtoRGB((float)hsl["hue"],1.0F, lightness);
				return System.Drawing.Color.FromArgb((int)rgb["red"], (int)rgb["green"],(int)rgb["blue"]);
			}
		}

		public System.Drawing.Color DarkColour
		{
			get
			{
				// decrease the lightness of the base colour
				Hashtable hsl = this.RGBtoHSL(this.BaseColour.R, this.BaseColour.G, this.BaseColour.B);
				Hashtable rgb = this.HSLtoRGB((float)hsl["hue"],(float)hsl["saturation"],0.3F);
				return System.Drawing.Color.FromArgb((int)rgb["red"], (int)rgb["green"],(int)rgb["blue"]);
			}
		}

		public System.Drawing.Color PresenceColour
		{
			get
			{
				return this.BaseColour;
				// adjust the hue of the base colour slightly
				//				Hashtable hsl = this.RGBtoHSL(this.newColour.R, this.newColour.G, this.newColour.B);
				//				Hashtable rgb = this.HSLtoRGB(((float)hsl["hue"])+15F,(float)hsl["saturation"], (float)hsl["lightness"]);
				//				return System.Drawing.Color.FromArgb((int)rgb["red"], (int)rgb["green"], (int)rgb["blue"]);
			}
		}

		public System.Drawing.Color AwayColour
		{
			get
			{
				// decrease the saturation of the presence colour
				Hashtable hsl = this.RGBtoHSL(this.PresenceColour.R, this.PresenceColour.G, this.PresenceColour.B);
				Hashtable rgb = this.HSLtoRGB((float)hsl["hue"], 0.3F*(float)hsl["saturation"], 0.8F*(float)hsl["lightness"]);
				return System.Drawing.Color.FromArgb((int)rgb["red"], (int)rgb["green"],(int)rgb["blue"]);
			}
		}

		private Hashtable HSLtoRGB(float hue, float saturation, float lightness)
		{
			Hashtable result = new Hashtable();

			if(saturation == 0)
			{
				result["red"] = (int)(lightness*255F);
				result["blue"] = (int)(lightness*255F);
				result["green"] = (int)(lightness*255F);
			}
			else
			{
				float temp1, temp2, temp3r, temp3g, temp3b;

				if(lightness < 0.5)
				{
					temp2 = lightness*(1F+saturation);
				}
				else
				{
					temp2 = lightness+saturation - lightness*saturation;
				}

				temp1 = 2F*lightness - temp2;

				hue = hue/360F;
				
				temp3r = hue+1F/3F;
				if(temp3r < 0) { temp3r += 1F; }
				if(temp3r > 1) { temp3r -= 1F; }
				temp3g = hue;
				if(temp3g < 0) { temp3g += 1F; }
				if(temp3g > 1) { temp3g -= 1F; }
				temp3b = hue-1F/3F;
				if(temp3b < 0) { temp3b += 1F; }
				if(temp3b > 1) { temp3b -= 1F; }
				
				if(6F*temp3r < 1)
				{
					result["red"] = temp1+(temp2-temp1)*6F*temp3r;
				}
				else if(2F*temp3r < 1) 
				{
					result["red"] = temp2;
				}
				else if(3F*temp3r < 2)
				{
					result["red"] = temp1+(temp2-temp1)*((2F/3F)-temp3r)*6F;
				}
				else
				{
					result["red"] = temp1;
				}

				if(6F*temp3g < 1)
				{
					result["green"] = temp1+(temp2-temp1)*6F*temp3g;
				}
				else if(2F*temp3g < 1) 
				{
					result["green"] = temp2;
				}
				else if(3F*temp3g < 2)
				{
					result["green"] = temp1+(temp2-temp1)*((2F/3F)-temp3g)*6F;
				}
				else
				{
					result["green"] = temp1;
				}

				if(6F*temp3b < 1)
				{
					result["blue"] = temp1+(temp2-temp1)*6F*temp3b;
				}
				else if(2F*temp3b < 1) 
				{
					result["blue"] = temp2;
				}
				else if(3F*temp3b < 2)
				{
					result["blue"] = temp1+(temp2-temp1)*((2F/3F)-temp3b)*6F;
				}
				else
				{
					result["blue"] = temp1;
				}
				result["red"] = (int)((float)result["red"]*255F);
				result["green"] = (int)((float)result["green"]*255F);
				result["blue"] = (int)((float)result["blue"]*255F);
			}
			return result;
		}

		private Hashtable RGBtoHSL(int red, int green, int blue)
		{
			Hashtable result = new Hashtable();
			
			float fred = (float)red/255F;
			float fgreen = (float)green/255F;
			float fblue = (float)blue/255F;

			float max = this.Max(fred, fgreen, fblue);
			float min = this.Min(fred, fgreen, fblue);

			result["lightness"] = (max+min)/2;

			if(max == min)
			{
				result["hue"] = 0;
				result["saturation"] = 0;
			}
			else
			{
				if(fred == max)
				{
					result["hue"] = ((fgreen-fblue)/(max-min))*60F;
				}
				else if(fgreen == max)
				{
					result["hue"] = (2F + (fblue-fred)/(max-min))*60F;
				}
				else
				{
					result["hue"] = (4F + (fred-fgreen)/(max-min))*60F;
				}
				if((float)result["hue"] < 0)
				{
					result["hue"] = (float)result["hue"] + 360F;
				}
				if((float)result["lightness"] >= 0.5)
				{
					result["saturation"] = (max-min)/(2F-max-min);
				}
				else
				{
					result["saturation"] = (max-min)/(max+min);
				}
			}
			return result;
		}

		private float Max(float first, float second, float third)
		{
			return Math.Max(first, Math.Max(second, third));
		}

		private float Min(float first, float second, float third)
		{
			return Math.Min(first, Math.Min(second, third));
		}

		private void CreateBaseColour()
		{
			ArrayList a = new ArrayList();
			a.Add(System.Drawing.Color.FromArgb(203,229,255));
			a.Add(System.Drawing.Color.FromArgb(203,255,203));
			a.Add(System.Drawing.Color.FromArgb(203,255,229));
			a.Add(System.Drawing.Color.FromArgb(203,255,255));
			a.Add(System.Drawing.Color.FromArgb(220,238,220));
			a.Add(System.Drawing.Color.FromArgb(220,238,238));
			a.Add(System.Drawing.Color.FromArgb(229,203,255));
			a.Add(System.Drawing.Color.FromArgb(229,255,203));
			a.Add(System.Drawing.Color.FromArgb(238,220,238));
			a.Add(System.Drawing.Color.FromArgb(238,238,220));
			a.Add(System.Drawing.Color.FromArgb(255,203,204));
			a.Add(System.Drawing.Color.FromArgb(255,229,203));
			a.Add(System.Drawing.Color.FromArgb(255,255,203));

			System.Random rand = new Random();
			this.baseColour = (System.Drawing.Color)a[rand.Next(a.Count)];
		}

		private int ColourValueForNumber(int number)
		{
			int colourValue;
			if(number == 0)
			{
				// FF
				colourValue = 255;
			}
			else if(number == 1)
			{
				// CC
				colourValue = 204;
			}
			else
			{
				// 153
				colourValue = 153;
			}
			return colourValue;
		}

		//========================================
		// Tests
		//========================================

		private void TestHSLtoRGB()
		{
			Hashtable test = new Hashtable();

			System.Diagnostics.Debug.WriteLine("-------------------------");

			test = this.HSLtoRGB(20F/239F*360F, 240F/240F, 120F/240F);
			System.Diagnostics.Debug.WriteLine(test["red"].ToString());
			System.Diagnostics.Debug.WriteLine(test["green"].ToString());
			System.Diagnostics.Debug.WriteLine(test["blue"].ToString());

			System.Diagnostics.Debug.WriteLine("");

			test = this.HSLtoRGB(133F/239F*360F, 1F, 90F/240F);
			System.Diagnostics.Debug.WriteLine(test["red"].ToString());
			System.Diagnostics.Debug.WriteLine(test["green"].ToString());
			System.Diagnostics.Debug.WriteLine(test["blue"].ToString());

			System.Diagnostics.Debug.WriteLine("");

			test = this.HSLtoRGB(200F/239F*360F, 240F/240F, 30F/240F);
			System.Diagnostics.Debug.WriteLine(test["red"].ToString());
			System.Diagnostics.Debug.WriteLine(test["green"].ToString());
			System.Diagnostics.Debug.WriteLine(test["blue"].ToString());
		
			System.Diagnostics.Debug.WriteLine("");

			test = this.HSLtoRGB(120F/239F*360F, 80F/240F, 90F/240F);
			System.Diagnostics.Debug.WriteLine(test["red"].ToString());
			System.Diagnostics.Debug.WriteLine(test["green"].ToString());
			System.Diagnostics.Debug.WriteLine(test["blue"].ToString());

		}

		private void TestRGBtoHSL()
		{
			Hashtable test = new Hashtable();

			System.Diagnostics.Debug.WriteLine("---------------------------");

			test = this.RGBtoHSL(255, 128, 0);
			System.Diagnostics.Debug.WriteLine(((float)test["hue"]/360F*240F).ToString());
			System.Diagnostics.Debug.WriteLine(((float)test["saturation"]*240F).ToString());
			System.Diagnostics.Debug.WriteLine(((float)test["lightness"]*240F).ToString());

			System.Diagnostics.Debug.WriteLine("");

			test = this.RGBtoHSL(0, 128, 192);
			System.Diagnostics.Debug.WriteLine(((float)test["hue"]/360F*240F).ToString());
			System.Diagnostics.Debug.WriteLine(((float)test["saturation"]*240F).ToString());
			System.Diagnostics.Debug.WriteLine(((float)test["lightness"]*240F).ToString());
		
			System.Diagnostics.Debug.WriteLine("");

			test = this.RGBtoHSL(64, 0, 64);
			System.Diagnostics.Debug.WriteLine(((float)test["hue"]/360F*240F).ToString());
			System.Diagnostics.Debug.WriteLine(((float)test["saturation"]*240F).ToString());
			System.Diagnostics.Debug.WriteLine(((float)test["lightness"]*240F).ToString());
		
			System.Diagnostics.Debug.WriteLine("");

			test = this.RGBtoHSL(64, 128, 128);
			System.Diagnostics.Debug.WriteLine(((float)test["hue"]/360F*240F).ToString());
			System.Diagnostics.Debug.WriteLine(((float)test["saturation"]*240F).ToString());
			System.Diagnostics.Debug.WriteLine(((float)test["lightness"]*240F).ToString());
		}
	}
}
