using System;
using System.Net;
using System.Net.Sockets;
using SideItem;
using GroupLab.Networking; 
using System.Collections; 
using System.Threading;
using ColourGenerator;
using System.Drawing;


namespace ItemTemplate
{
	/// <summary>
	/// Summary description for ItemTemplate.
	/// </summary>
	/// 
	
	[SideItem.ItemName("Aibo Awareness")] 
	public class ItemTemplate : SideItem.IItem 
	{
		// These required variables  will be assigned in the StartItem method
		private GroupLab.Networking.SharedDictionary sd;
		private string key;
		private bool master;

		// These optional variables  are often useful to have
		private string currentUserName; //the current user's name
		private string ownerName;       //the owner of this item's name

		// These variables give us access to our tile, transient and large window
		// if you don't need a separate master/slave Tile, just remove the 
		//masterTile/SlaveTile declarations and use
		public Tile Tile;	// a tile user control
		public Image currImage;
		
		ArrayList transients;  // a transient; as more than one can be raised, we have to create it as an array
		ArrayList separates;

		// the colour reference to use the colour scheme from the Place
		ColourGenerator.ColourGenerator colour;

		// Variables for networking
		Thread startClient;
		private UdpClient UDPCamera;
		private const int cameraPort = 10011;
		private IPEndPoint cameraRP;

		private TcpClient TCPcommand;
		private const int commandPort = 10020;
		
		private TcpClient TCPheadPoint;
		private const int headPointPort = 10052;
		public const char HPTilt = 't';
		public const char HPPan = 'p';
		public const char HPRoll = 'r';
		
		private TcpClient TCPwalk;
		private const int walkPort = 10050;
		public const char WALKFwd = 'f';
		public const char WALKRoto = 'r';
		public const char WALKHowl = '5';
		public const char WALKYap = '6';
		public const char WALKWhimper = '7';
		public const char WALKGrowl = '8';
		public const char WALKBark = '9';

		private const string ip = "192.168.24.169";
		private bool GrabVideo;
		private const int CONNECTION_TRIES = 5;

		private const string CMDToggleVision = "!root \"TekkotsuMon\" \"Raw Cam Server\"\r\n";
		private const string CMDToggleHeadPoint = "!root \"TekkotsuMon\" \"Head Remote Control\"\r\n";
		private const string CMDToggleWalk = "!root \"TekkotsuMon\" \"Walk Remote Control\"\r\n";

		//Subscriptions
		private GroupLab.Networking.Subscription imgSub;
		private const string imgKey = "/Image";
		private const string imgKeyImage = "Image";
		private const string imgKeyHeight = "Height";
		private const string imgKeyWidth = "Width";

		private GroupLab.Networking.Subscription HPSub;
		private const string HPKey = "/HeadPoint";
		private GroupLab.Networking.Map HPMap;
		
		private GroupLab.Networking.Subscription WALKSub;
		private const string WALKKey = "/Walk";
		private GroupLab.Networking.Map WALKMap;
	
		// IN USE timer and subscription
		private System.Timers.Timer InUseTimer;
		private const int InUseMAXTicks = 10;
		private int InUseTicks = 0;
		private const int InUseTickLength = 100;
		
		private GroupLab.Networking.Subscription InUseSub;
		private const string InUseKey = "/InUse";

		public ItemTemplate()
		{
			// if you don't need a separate master/slave Tile, just remove the 
			//masterTile/SlaveTile assignments and use 
			this.Tile = new Tile(this); // create a new tile user control
			this.transients = new ArrayList(); // create a new array to hold the transients
			this.separates = new ArrayList();

			this.HPMap = new GroupLab.Networking.Map();
			this.WALKMap = new GroupLab.Networking.Map();

			InUseTimer = new System.Timers.Timer();
			InUseTimer.Interval = InUseTickLength;
			InUseTimer.Elapsed += new System.Timers.ElapsedEventHandler(InUseTick);
		}
		#region IItem Members

		public void SeparateItem()
		{
			// if you don't need a separate master/slave Tile, just remove the if/else below and call 
			Separate ms = new Separate(this);
			ms.Show();
			ms.Disposed += new EventHandler(this.s_Disposed);
			this.separates.Add(ms);
		}

		public System.Windows.Forms.Control GetTransientFocus(System.Drawing.Size size)
		{
			Transient t;
			t = new Transient (this);
			t.Size = size;
			t.Disposed += new EventHandler(this.t_Disposed);
			this.transients.Add(t);
			return t;
		}

		private void _Disposed(object sender, EventArgs e)
		{
			this.StopItem();
		}

		public void StopItem()
		{
			if (this.master) 
			{
				GrabVideo = false;
				HPSub.Notified -= new SubscriptionEventHandler(this.ControlHPNotified);
				WALKSub.Notified -= new SubscriptionEventHandler(this.ControlWALKNotified);
				if (InUseTimer.Enabled)
					InUseTimer.Stop();
				startClient.Join();
				SendCommand(CMDToggleVision);
				SendCommand(CMDToggleHeadPoint);
				SendCommand(CMDToggleWalk);
				if (TCPcommand != null) 
					TCPcommand.Close();
				if (TCPheadPoint != null)
					TCPheadPoint.Close();
				if (UDPCamera != null)
					UDPCamera.Close();
			} 
			else 
			{
				imgSub.Notified -= new SubscriptionEventHandler(this.ImageNotified);
				InUseSub.Notified -= new SubscriptionEventHandler(this.InUseNotified);
			}
		}

		public event SideItem.ChangeEventHandler ItemChanged;

		// return the tile user control
		public System.Windows.Forms.Control GetTile(int focus)
		{
			return this.Tile;			
		}

		public void StartItem(GroupLab.Networking.SharedDictionary sd, string path, bool master, string currentUserKey)
		{
			this.sd = sd;
			this.key = path;
			this.master = master;

			// get the current user's name
			this.currentUserName = (string)this.sd[currentUserKey];

			string placeKey = (this.sd[this.key] as Map)["place"] as String;
			GroupLab.Networking.Map seedcolour = sd[placeKey+"/colour"] as GroupLab.Networking.Map;
			this.colour = new ColourGenerator.ColourGenerator(System.Drawing.Color.FromArgb((int)seedcolour["red"], (int)seedcolour["green"], (int)seedcolour["blue"]));

			// get the owner's name - we get the user from the item information
			GroupLab.Networking.Map itemData = (GroupLab.Networking.Map)this.sd[this.key];
			string ownerKey = (string)itemData["owner"];
			this.ownerName = (string)this.sd[ownerKey];

			// subscribe to your shared data
			this.Tile.Disposed += new EventHandler(this._Disposed);
			
			if (this.master) 
			{
				this.HPSub = new GroupLab.Networking.Subscription();
				this.HPSub.BeginInit();
				this.HPSub.Dictionary = this.sd;
				this.HPSub.Pattern = this.key + HPKey;
				this.HPSub.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.ControlHPNotified);
				this.HPSub.EndInit();

				this.WALKSub = new GroupLab.Networking.Subscription();
				this.WALKSub.BeginInit();
				this.WALKSub.Dictionary = this.sd;
				this.WALKSub.Pattern = this.key + WALKKey;
				this.WALKSub.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.ControlWALKNotified);
				this.WALKSub.EndInit();

				SetupAIBO();
			}

			this.imgSub = new GroupLab.Networking.Subscription();
			this.imgSub.BeginInit();
			this.imgSub.Dictionary = this.sd;
			this.imgSub.Pattern = this.key + imgKey;
			this.imgSub.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.ImageNotified);
			this.imgSub.EndInit();

			this.InUseSub = new GroupLab.Networking.Subscription();
			this.InUseSub.BeginInit();
			this.InUseSub.Dictionary = this.sd;
			this.InUseSub.Pattern = this.key + InUseKey;
			this.InUseSub.Notified += new GroupLab.Networking.SubscriptionEventHandler(this.InUseNotified);
			this.InUseSub.EndInit();
		}
		#endregion

		// Get the colour generator for the Place that the item is in.
		// - Colour.AwayColour is greyer (reduced saturation)
		// - Colour.NewColour is brighter (increased saturation)
		// - Colour.DarkColour is a dark version of the colour (decreased lightness)
		public ColourGenerator.ColourGenerator Colour
		{
			get
			{
				return this.colour;
			}
		}

		// To dispose the transient:
		// transients will appear and disappear all the time 
		// and when they are gone they should be removed from the ArrayList.
		private void t_Disposed(object sender, EventArgs e)
		{
			this.transients.Remove(sender);
		}

		private void s_Disposed(object sender, EventArgs e)
		{
			this.separates.Remove(sender);
		}

		
		private void ResetTimer() 
		{
			InUseTicks = 0;
			if (InUseTimer.Enabled == false) 
			{
				InUseTimer.Start();
				sd[this.key + InUseKey] = true;
			}
		}

		#region Callbacks
	
		private void InUseTick(object sender, System.Timers.ElapsedEventArgs e) 
		{
			InUseTicks++;
			if (InUseTicks > InUseMAXTicks) 
			{
				InUseTimer.Stop();
				sd[this.key + InUseKey] = false;
			}
		}

		private void ControlHPNotified(object sender, GroupLab.Networking.SubscriptionEventArgs e) 
		{
			if (GroupLab.Networking.SubscriptionNotification.AttributeChange == e.Reason) 
			{
				float amount = (float)this.sd[this.key + HPKey +"."+ e.AttributeName];
				SendHPCommand(e.AttributeName[1], amount);
				ResetTimer();
			}	
					
		}

		private void ControlWALKNotified(object sender, GroupLab.Networking.SubscriptionEventArgs e) 
		{
			if (GroupLab.Networking.SubscriptionNotification.AttributeChange == e.Reason) 
			{
				float amount=(float)this.sd[this.key + WALKKey + "." + e.AttributeName];
				SendWalkCommand(e.AttributeName[1],amount);
				ResetTimer();
			}	
					
		}

		private void InUseNotified(object sender, GroupLab.Networking.SubscriptionEventArgs e) 
		{
			if (e.Reason == GroupLab.Networking.SubscriptionNotification.Change) 
			{
				this.Tile.InUse( (bool)sd[this.key + InUseKey]);
			}
		}

		private void ImageNotified(object sender, GroupLab.Networking.SubscriptionEventArgs e) 
		{
			if (GroupLab.Networking.SubscriptionNotification.Change == e.Reason)
			{
				if (this.master) 
				{
					; // Nothing special
				} 
				else 
				{
					GroupLab.Networking.Map m = new GroupLab.Networking.Map();
					m = this.sd[this.key + imgKey] as GroupLab.Networking.Map;

					System.IO.MemoryStream str = (System.IO.MemoryStream) m[imgKeyImage];
					this.currImage = Image.FromStream(str, false, false);
				}
				UpdateImage(this.currImage);
			}		
		}

		private void UpdateImage(System.Drawing.Image image) 
		{
			this.Tile.SetImage((Image)image.Clone());
			
			foreach (Transient t in this.transients) 
			{
				t.SetImage((Image)image.Clone());
			}

			foreach (Separate s in this.separates)
			{
				s.SetImage((Image)image.Clone());
			}
		}

		#endregion

		#region Talk to AIBO code

		private void SendCommand(string command) 
		{
			Byte[] sendBytes = System.Text.Encoding.ASCII.GetBytes(command);
			NetworkStream ns = TCPcommand.GetStream();
			ns.Write(sendBytes, 0, sendBytes.Length);
			ns.Flush();
		}

		public void SendHPCommand(char HPType, float amount) 
		{
			if (this.master) 
			{
				Byte[] sendBytes = new Byte[5];
				sendBytes[0] = (byte)HPType;
				Byte[] floatB = System.BitConverter.GetBytes(amount);
			
				for (int i=0; i <floatB.Length;i++)
					sendBytes[i+1] = floatB[i];
				NetworkStream ns = TCPheadPoint.GetStream();
				ns.Write(sendBytes, 0, sendBytes.Length);
				ns.Flush();
				ResetTimer();
			} 
			else 
			{
				sd[this.key + HPKey + "." + HPType.ToString()] = amount;
			}
		}

		public void SendWalkCommand(char WalkType, float amount) 
		{
			if (this.master) 
			{
				Byte[] sendBytes = new Byte[5];
				sendBytes[0] = (byte)WalkType;
				Byte[] floatB = System.BitConverter.GetBytes(amount);
			
				for (int i=0; i <floatB.Length;i++)
					sendBytes[i+1] = floatB[i];
				NetworkStream ns = TCPwalk.GetStream();
				ns.Write(sendBytes, 0, sendBytes.Length);
				ns.Flush();
				ResetTimer();
			} 
			else 
			{
				sd[this.key + WALKKey + "." + WalkType.ToString()] = amount;
			}
		}

		enum Phase 
		{
			TCPConnect,
			VisionStart,
			VisionTest,
			HeadPointStart,
			WalkStart,
			Done
		};

		private bool ConnectToAIBO() 
		{
			int tries= 0;
			bool connected = false;

			Phase phase = Phase.TCPConnect;  
			while (phase != Phase.Done && tries <= CONNECTION_TRIES) 
			{
				tries += 1;
				try
				{
					if (phase == Phase.TCPConnect) 
					{
						TCPcommand = new TcpClient(ip, commandPort);
						phase = Phase.VisionStart;
						tries = 0;
					} 
					else if (phase == Phase.VisionStart) 
					{
						SendCommand(CMDToggleVision);
						Thread.Sleep(500);
						cameraRP = new IPEndPoint(IPAddress.Parse(ip), cameraPort);
						UDPCamera = new UdpClient(ip, cameraPort);
						Byte[] dgram = System.Text.Encoding.ASCII.GetBytes("connection request");
						UDPCamera.Send(dgram, dgram.Length);
						tries = 0;
						phase = Phase.VisionTest;
					} 
					else if (phase == Phase.VisionTest) 
					{
						try 
						{
							byte[] recData = UDPCamera.Receive(ref cameraRP);
							tries = 0;
							phase = Phase.HeadPointStart;
						} 
						catch (Exception e) 
						{
							phase = Phase.VisionStart;
						}
					}
					else if (phase == Phase.HeadPointStart) 
					{
						SendCommand(CMDToggleHeadPoint);
						Thread.Sleep(500);

						TCPheadPoint = new TcpClient(ip, headPointPort);

						SendHPCommand(HPTilt, 0);
						SendHPCommand(HPRoll, 0);
						SendHPCommand(HPPan, 0);

						phase = Phase.WalkStart;
						tries = 0;
					} 
					else if (phase == Phase.WalkStart) 
					{
						SendCommand(CMDToggleWalk);
						Thread.Sleep(500);

						TCPwalk = new TcpClient(ip, walkPort);

						SendWalkCommand(WALKFwd, 0);
						SendWalkCommand(WALKRoto, 0);
						
						phase = Phase.Done;
						tries = 0;
					} 
				}
				catch (Exception e) 
				{
					Console.WriteLine(e.ToString());
				}	
			}

			return (phase == Phase.Done);	
		}

		private bool SetupAIBO() 
		{
			if (!ConnectToAIBO()) return false;
			GrabVideo = true;
			
			startClient = new Thread(new ThreadStart(start_client));
			startClient.Start();
			return true;
		}
		
		private void start_client() 
		{
			
			System.Text.ASCIIEncoding encode = new System.Text.ASCIIEncoding();
			while (GrabVideo)
			{
				try 
				{
					byte[] recData = UDPCamera.Receive(ref cameraRP);
					ParsePacket(recData);
				}
				catch (Exception e) 
				{
					Console.WriteLine(e.ToString());
				}
			}
		}

		uint getInt(byte [] input, int index) 
		{
			return ( (uint)(
				(byte)input[0+index] |
				(byte)input[1+index] << 8 |
				(byte)input[2+index] << 16 |
				(byte)input[3+index] << 24
			));
		}	

		class AIBOPacket
		{
			
			public uint nameLen;
			public string name;
			public uint imageEncoding;
			public uint imageCompression;
			public uint imageWidth;
			public uint imageHeight;
			public uint imageTimestamp;
			public uint imageFramenumber;

			// Generator info
			public uint generatorNameLen;
			public string generatorName;
			public uint generatorImageWidth;
			public uint generatorImageHeight;
			public uint generatorImageLayer;
			public uint generatorImageChannel;
	
			public uint specificGenNameLen;
			public string specificGenName;
			public uint jpegImageDataSize;

		}

		void ParsePacket( byte[] packet) 
		{
			int index = 0;
			AIBOPacket ap = new AIBOPacket();

			ap.nameLen = getInt(packet, index);  index += 4;
			ap.name = System.Text.ASCIIEncoding.ASCII.GetString(packet,index,(int)ap.nameLen);
			index += (int)ap.nameLen+1;
			ap.imageEncoding = getInt(packet, index); index += 4;
			ap.imageCompression = getInt(packet, index); index += 4;
			ap.imageWidth = getInt(packet, index); index += 4;
			ap.imageHeight = getInt(packet, index); index += 4;
			ap.imageTimestamp = getInt(packet, index); index += 4;
			ap.imageFramenumber = getInt(packet, index); index+= 4;
	
			ap.generatorNameLen = getInt(packet, index); index+= 4;
			ap.generatorName = System.Text.ASCIIEncoding.ASCII.GetString(packet,index,
								(int)ap.generatorNameLen);
			index += (int)ap.generatorNameLen+1;

			ap.generatorImageWidth = getInt(packet, index); index += 4;
			ap.generatorImageHeight = getInt(packet, index); index += 4;
			ap.generatorImageLayer = getInt(packet, index); index += 4;
			ap.generatorImageChannel = getInt(packet, index); index += 4;

			ap.specificGenNameLen = getInt(packet, index); index += 4;
			ap.specificGenName = System.Text.ASCIIEncoding.ASCII.GetString(packet,index,
								   (int)ap.specificGenNameLen);
			index += (int)ap.specificGenNameLen+1;

			ap.jpegImageDataSize = getInt(packet,index); index += 4;
				
			System.IO.MemoryStream str = new System.IO.MemoryStream(packet, index, packet.Length-index, true, true);
			
			GroupLab.Networking.Map m = new GroupLab.Networking.Map();
			m[imgKeyImage] = str;
			m[imgKeyWidth] = ap.imageWidth;
			m[imgKeyHeight] = ap.imageHeight;
			this.sd[this.key + imgKey] = m;

			this.currImage = Image.FromStream(str, false, false);
//			UpdateImage(Image.FromStream(str,false,false));
		}

		#endregion
	}
}
