using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace ItemTemplate
{
	/// <summary>
	/// Summary description for Tile.
	/// </summary>
	public class Tile : System.Windows.Forms.UserControl
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.PictureBox pictureBox1;
		private float photoRatio = 1;
		private System.Windows.Forms.Label title;
		private System.Windows.Forms.CheckBox activeBox;
		private string ActiveCaption = "In Use";
		private int CheckWidth = 15;
		private int ImageMinHeight = 40;
		
		private enum Mode 
		{
			Full,
			NoActiveCaption,
			NoImageButCaption,
			TitleOnly
		};

		private Mode DisplayMode = Mode.Full;

		// a pointer back to the main object 
		// don't forget to change ItemTemplate to whatever you call it!
		protected ItemTemplate main; 

		public Tile()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}

		public Tile(ItemTemplate main)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			this.main = main;
		
			// You can delete this.
			this.activeBox.Text = ActiveCaption;
			this.DisplayMode = Mode.Full;
			//this.BackColor = main.Colour.BaseColour;
		}

		#region Dispose
		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.pictureBox1 = new System.Windows.Forms.PictureBox();
			this.title = new System.Windows.Forms.Label();
			this.activeBox = new System.Windows.Forms.CheckBox();
			this.SuspendLayout();
			// 
			// pictureBox1
			// 
			this.pictureBox1.Location = new System.Drawing.Point(16, 40);
			this.pictureBox1.Name = "pictureBox1";
			this.pictureBox1.SizeMode = System.Windows.Forms.PictureBoxSizeMode.StretchImage;
			this.pictureBox1.TabIndex = 0;
			this.pictureBox1.TabStop = false;
			// 
			// title
			// 
			this.title.BackColor = System.Drawing.Color.Peru;
			this.title.Font = new System.Drawing.Font("Microsoft Sans Serif", 8F, System.Drawing.FontStyle.Bold);
			this.title.Location = new System.Drawing.Point(0, 0);
			this.title.Name = "title";
			this.title.Size = new System.Drawing.Size(192, 12);
			this.title.TabIndex = 1;
			this.title.Text = "AiboAware";
			// 
			// activeBox
			// 
			this.activeBox.CheckAlign = System.Drawing.ContentAlignment.MiddleRight;
			this.activeBox.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.activeBox.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.activeBox.Location = new System.Drawing.Point(0, 138);
			this.activeBox.Name = "activeBox";
			this.activeBox.Size = new System.Drawing.Size(150, 12);
			this.activeBox.TabIndex = 2;
			this.activeBox.Text = "In Use";
			this.activeBox.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.activeBox.CheckedChanged += new System.EventHandler(this.activeBox_CheckedChanged);
			// 
			// Tile
			// 
			this.Controls.Add(this.title);
			this.Controls.Add(this.pictureBox1);
			this.Controls.Add(this.activeBox);
			this.Name = "Tile";
			this.SizeChanged += new System.EventHandler(this.Tile_SizeChanged);
			this.ResumeLayout(false);

		}
		#endregion

		//Code copied from Saul's photoitem media item.
		public System.Drawing.Point ImageResize(int AdjHeight, bool set) 
		{
			// Resize to fit the tile while maintaining ratio
			System.Drawing.Point pt = new System.Drawing.Point(0,0);

			// Try to make image match the width
			pt.Y = (int)(this.Width*photoRatio);
			if (pt.Y < AdjHeight) 
			{
				//it worked
				pt.X = this.Width;
			} 
			else
			{
				pt.X =(int)((float)AdjHeight*(1/photoRatio));
				pt.Y = AdjHeight;
			}
			if (set) 
			{
				this.pictureBox1.Height = pt.Y;
				this.pictureBox1.Width = pt.X;
			}
			return pt;
		}

		public void SetImage(System.Drawing.Image image) 
		{
			photoRatio = (float)(image.Height) /
				(float)(image.Width);
			this.pictureBox1.Image = image;
			DrawTile();
		}

		public void TileResize(int focusHeight) 
		{
			this.Height = focusHeight;
			DrawTile();
		}

		public void DrawTile() 
		{
			int AdjHeight = this.Height - this.title.Height;

			if (DisplayMode == Mode.Full) 
			{
				AdjHeight -= this.activeBox.Height;
				ImageResize(AdjHeight, true);
				
				if ((this.Width - this.pictureBox1.Width)/2 >= CheckWidth)
				{	
					activeBox.Text = "";
					DisplayMode = Mode.NoActiveCaption;
				}
			} 
			
			if (DisplayMode == Mode.NoActiveCaption) 
			{
				System.Drawing.Point pt = ImageResize(AdjHeight-this.activeBox.Height, false);
				activeBox.Text = "";
				if ( (this.Width - pt.X)/2 <= CheckWidth - 4)
				{
					DisplayMode = Mode.Full;
					activeBox.Text = ActiveCaption;
				} 
				else if (this.Height < ImageMinHeight) 
				{
					DisplayMode = Mode.NoImageButCaption;
					
					activeBox.Text = ActiveCaption;
					this.pictureBox1.Visible = false;
				}
			}

			if (DisplayMode == Mode.NoImageButCaption)
			{
				if (this.Height >= ImageMinHeight) 
				{
					DisplayMode = Mode.NoActiveCaption;
					activeBox.Text = "";
					this.pictureBox1.Visible = true;
				} 
				else if (this.Height <= (this.title.Height+this.activeBox.Height))
				{
					activeBox.Visible = false;
					DisplayMode = Mode.TitleOnly;
				}

			}
			else if (DisplayMode == Mode.TitleOnly)
			{
				if (this.Height >= (this.title.Height+this.activeBox.Height))
				{
					DisplayMode = Mode.NoImageButCaption;
					activeBox.Visible = true;
				}
			}

			this.pictureBox1.Top = (AdjHeight)/2 - this.pictureBox1.Height/2 + this.title.Height;
			this.pictureBox1.Left = this.Width/2 - this.pictureBox1.Width/2;
		}

		
		private void Tile_SizeChanged(object sender, System.EventArgs e)
		{
			DrawTile();
		}

		private void activeBox_CheckedChanged(object sender, System.EventArgs e)
		{
		
		}

		public void InUse(bool use) 
		{
			this.activeBox.Checked = use;
		}

	}
}
