using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace ItemTemplate
{
	/// <summary>
	/// Summary description for Transient.
	/// </summary>
	public class Transient : System.Windows.Forms.UserControl
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.PictureBox pictureBox1;

		private bool Looking = false;
		private bool Walking = false;

		private System.Windows.Forms.TrackBar trackBar1;
		private System.Windows.Forms.Label lblNeck;
		private System.Windows.Forms.PictureBox picWalk;
		private int controlSpacer = 5;
		Graphics walkGraph;
		private int walkSize = 50;
		private float walkTurnScale = 0.5F;
		System.Drawing.Point walkPt;  // Relative to center, 0..1
		System.Drawing.Pen walkPen;
		private int AIBOWidth = 20;  // set
		private int AIBOHeight;  // Calculated

		Graphics videoGraph;
		Image videoImage;
		System.Drawing.Point lookPt;  // Relative to top/left
		bool lookPtExists = false;
		int lookPtSize = 5;
		System.Drawing.Pen lookPtPen;
		System.Drawing.Pen crossHairPen;
		private System.Windows.Forms.PictureBox picAIBO;

		// a pointer back to the main object 
		// don't forget to change ItemTemplate to whatever you call it!
		protected ItemTemplate main; 

		public Transient()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}
		
		public Transient(ItemTemplate main)
		{
			
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			this.main = main;

			// You can delete this.
			//this.BackColor = System.Drawing.Color.Goldenrod;
			this.pictureBox1.Top = 0;
			this.pictureBox1.Left = 0;
			this.pictureBox1.SizeMode = PictureBoxSizeMode.Normal;
			this.pictureBox1.Width = 640;
			this.pictureBox1.Height = 480;
			
			videoGraph = this.pictureBox1.CreateGraphics();

			Looking = false;

			crossHairPen = new System.Drawing.Pen(System.Drawing.Color.FromArgb(127,27,200,27), 7);
			crossHairPen.DashPattern = new float[]{0.5F, 0.5F, 0.5F, 0.5F};
			
			lookPtPen = new System.Drawing.Pen(System.Drawing.Color.FromArgb(200,200,27,27), 2);

			walkPen = new System.Drawing.Pen(System.Drawing.Color.FromArgb(127,27,27,200), 7);
			walkPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash;
			walkPen.DashPattern = new float[]{0.5F, 0.5F, 0.5F, 0.5F};

			this.picWalk = new System.Windows.Forms.PictureBox();
			this.picWalk.MouseUp += new System.Windows.Forms.MouseEventHandler(this.picWalk_MouseUp);
			this.picWalk.MouseMove += new System.Windows.Forms.MouseEventHandler(this.picWalk_MouseMove);
			this.picWalk.MouseDown += new System.Windows.Forms.MouseEventHandler(this.picWalk_MouseDown);
			
			this.Controls.Add(this.picWalk);
			this.picWalk.SendToBack();
			this.picWalk.SizeMode = PictureBoxSizeMode.Normal;
			this.picWalk.Width = walkSize;
			this.picWalk.Height = walkSize;
			walkGraph = this.picWalk.CreateGraphics();

			Image I = picAIBO.Image;
			AIBOHeight = (int)( (float)I.Height/(float)I.Width*(float)AIBOWidth);
		}

		public void FormLayout() 
		{
			pictureBox1.Width = videoImage.Width;
			pictureBox1.Height = videoImage.Height;	
	
			picWalk.Top = pictureBox1.Bottom + controlSpacer;
			pictureBox1.Left = 0;
	
			this.Width = videoImage.Width + trackBar1.Width;
			this.Height = videoImage.Height + picWalk.Height + controlSpacer*2;
			
			picWalk.Left = this.pictureBox1.Width/2 - picWalk.Width/2;

			lblNeck.Left = trackBar1.Left = pictureBox1.Right;
			trackBar1.Height = pictureBox1.Height;

			walkPt = new Point(picWalk.Width/2, picWalk.Height/2);
		}

		public void DrawImage() 
		{
			if (videoImage == null)
				return;
			
			Graphics offscreen = Graphics.FromImage(videoImage);
			int midX = videoImage.Width / 2;
			int midY = videoImage.Height / 2;
			offscreen.DrawLine(crossHairPen, midX, midY, midX, 0);
			offscreen.DrawLine(crossHairPen, midX, midY, midX, videoImage.Height-1);
			offscreen.DrawLine(crossHairPen, midX, midY, 0, midY);
			offscreen.DrawLine(crossHairPen, midX, midY, videoImage.Width-1, midY);
			if (lookPtExists) 
				offscreen.DrawEllipse(lookPtPen, lookPt.X-lookPtSize, lookPt.Y-lookPtSize,
					                  lookPtSize*2, lookPtSize*2);
			videoGraph.DrawImage(videoImage, 0,0,videoImage.Width, videoImage.Height); 

			offscreen.Dispose();
			
			Bitmap walkImage = new Bitmap(picWalk.Width, picWalk.Height);
			midX = picWalk.Width /2;
			midY = picWalk.Height /2;
			offscreen = Graphics.FromImage(walkImage);
			offscreen.Clear(this.BackColor);
			offscreen.DrawLine(walkPen, midX, midY, midX, 0);
			offscreen.DrawLine(walkPen, midX, midY, midX, picWalk.Height);
			offscreen.DrawLine(walkPen, midX, midY, 0, midY);
			offscreen.DrawLine(walkPen, midX, midY, picWalk.Width, midY);
			offscreen.DrawRectangle(System.Drawing.Pens.Black,0,0,picWalk.Width-1,picWalk.Height-1);
			midX = AIBOWidth/2;
			midY = AIBOHeight/2;

			offscreen.DrawImage(picAIBO.Image, walkPt.X-midX,walkPt.Y-midY,
								AIBOWidth,AIBOHeight);
			walkGraph.DrawImage(walkImage, 0, 0, walkImage.Width, walkImage.Height);
			offscreen.Dispose();

		}

		private static int SettingImage = 0;
		public void SetImage(System.Drawing.Image image) 
		{
			//while (0 != System.Threading.Interlocked.Exchange(ref SettingImage, 1)) 
			{
					if (videoImage == null || image.Width != videoImage.Width) 
					{
						videoImage = image;
						FormLayout();	
					}
				// only update on size change
				videoImage = image;
				DrawImage();
				//System.Threading.Interlocked.Exchange(ref SettingImage, 0);
			}
		}

		#region Dispose
		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					walkPen.Dispose();
					crossHairPen.Dispose();
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Transient));
			this.pictureBox1 = new System.Windows.Forms.PictureBox();
			this.trackBar1 = new System.Windows.Forms.TrackBar();
			this.lblNeck = new System.Windows.Forms.Label();
			this.picAIBO = new System.Windows.Forms.PictureBox();
			((System.ComponentModel.ISupportInitialize)(this.trackBar1)).BeginInit();
			this.SuspendLayout();
			// 
			// pictureBox1
			//
			this.pictureBox1.Location = new System.Drawing.Point(8, 8);
			this.pictureBox1.Name = "pictureBox1";
			this.pictureBox1.Size = new System.Drawing.Size(136, 104);
			this.pictureBox1.TabIndex = 0;
			this.pictureBox1.TabStop = false;
			this.pictureBox1.Paint += new System.Windows.Forms.PaintEventHandler(this.pictureBox1_Paint);
			this.pictureBox1.MouseUp += new System.Windows.Forms.MouseEventHandler(this.pictureBox1_MouseUp);
			this.pictureBox1.MouseMove += new System.Windows.Forms.MouseEventHandler(this.pictureBox1_MouseMove);
			this.pictureBox1.MouseLeave += new System.EventHandler(this.pictureBox1_MouseLeave);
			this.pictureBox1.MouseDown += new System.Windows.Forms.MouseEventHandler(this.pictureBox1_MouseDown);
			// 
			// trackBar1
			// 
			this.trackBar1.LargeChange = 20;
			this.trackBar1.Location = new System.Drawing.Point(144, 8);
			this.trackBar1.Maximum = 100;
			this.trackBar1.Name = "trackBar1";
			this.trackBar1.Orientation = System.Windows.Forms.Orientation.Vertical;
			this.trackBar1.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.trackBar1.Size = new System.Drawing.Size(42, 120);
			this.trackBar1.TabIndex = 1;
			this.trackBar1.TickFrequency = 10;
			this.trackBar1.Value = 100;
			this.trackBar1.ValueChanged += new System.EventHandler(this.trackBar1_ValueChanged);
			// 
			// lblNeck
			// 
			this.lblNeck.BackColor = System.Drawing.Color.Transparent;
			this.lblNeck.Location = new System.Drawing.Point(144, 0);
			this.lblNeck.Name = "lblNeck";
			this.lblNeck.Size = new System.Drawing.Size(40, 12);
			this.lblNeck.TabIndex = 2;
			this.lblNeck.Text = "Neck";
			// 
			// picAIBO
			// 
			this.picAIBO.Image = ((System.Drawing.Image)(resources.GetObject("picAIBO.Image")));
			this.picAIBO.Location = new System.Drawing.Point(0, 0);
			this.picAIBO.Name = "picAIBO";
			this.picAIBO.Size = new System.Drawing.Size(1, 1);
			this.picAIBO.TabIndex = 3;
			this.picAIBO.TabStop = false;
			this.picAIBO.Visible = false; 
			// 
			// Transient
			// 
			this.Controls.Add(this.picAIBO);
			this.Controls.Add(this.lblNeck);
			this.Controls.Add(this.pictureBox1);
			this.Controls.Add(this.trackBar1);
			this.Name = "Transient";
			this.Size = new System.Drawing.Size(184, 264);
			((System.ComponentModel.ISupportInitialize)(this.trackBar1)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		private void pictureBox1_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			Looking = true;
			SendLookCommand(e.X, e.Y);
			lookPt = new System.Drawing.Point(e.X, e.Y);
			lookPtExists = true;
		}

		private void picWalk_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			Walking = true;
			walkPt = new Point(e.X, e.Y);
			SendWalkCommand(e.X, e.Y);
		}

		private void SendLookCommand(int X, int Y) 
		{
			if (X < 0) X = 0;  if (Y < 0) Y = 0;
			if (X > pictureBox1.Width) X = pictureBox1.Width;
			if (Y > pictureBox1.Height) Y = pictureBox1.Height;

			float x = (float)(2*X - pictureBox1.Width)/(float)pictureBox1.Width*-1;
			float y = (float)(2*Y - pictureBox1.Height)/(float)pictureBox1.Height*-1;
			main.SendHPCommand(ItemTemplate.HPPan, x);
			main.SendHPCommand(ItemTemplate.HPRoll, y);
		}

		private void SendWalkCommand(int X, int Y) 
		{
			if (X < 0) X = 0;  if (Y < 0) Y = 0;
			if (X > picWalk.Width) X = picWalk.Width;
			if (Y > picWalk.Height) Y = picWalk.Height;

			float x = (float)(2*X - picWalk.Width)/(float)picWalk.Width*-1;
			float y = (float)(2*Y - picWalk.Height)/(float)picWalk.Height*-1;
			main.SendWalkCommand(ItemTemplate.WALKFwd, y);
			main.SendWalkCommand(ItemTemplate.WALKRoto, x*walkTurnScale);
		}

		private void picWalk_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (Walking) 
			{
				SendWalkCommand(e.X, e.Y);
				walkPt = new System.Drawing.Point(e.X, e.Y);
			}
		}

		private void picWalk_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			Walking = false;
			walkPt = new Point(picWalk.Width/2,picWalk.Height/2);
		}

		private void pictureBox1_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (Looking) 
			{
				SendLookCommand(e.X, e.Y);
				lookPt = new System.Drawing.Point(e.X, e.Y);
			}
		}

		private void pictureBox1_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			Looking = false;
		}

		private void pictureBox1_MouseLeave(object sender, System.EventArgs e)
		{
			Looking = false;
		}

		private void trackBar1_ValueChanged(object sender, System.EventArgs e)
		{
			System.Windows.Forms.TrackBar tb = (System.Windows.Forms.TrackBar)sender;
			float v = -1*(1-(float)tb.Value/(float)tb.Maximum);
			main.SendHPCommand(ItemTemplate.HPTilt, v);
		}

		private void pictureBox1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
/*			if (this.videoImage != null)
				SetImage(this.videoImage);*/
		}
	}
}
