using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;

namespace PresenceItem
{
	/// <summary>
	/// Summary description for IdleInputTracker.
	/// </summary>
	//[DefaultEvent("InputIdle")]
	public class IdleInputTracker
	{
		private System.Windows.Forms.Timer timer1;
		int triggerTime = 5 * 60 * 1000;
		bool triggered = false;

		public IdleInputTracker(int tickInterval, int timeout)
		{
			this.triggerTime = timeout;
			this.timer1 = new System.Windows.Forms.Timer();
			// 
			// timer1
			// 
			this.timer1.Interval = tickInterval;
			this.timer1.Tick += new System.EventHandler(this.timer1_Tick);
			this.timer1.Enabled = true;
		}

		private void timer1_Tick(object sender, System.EventArgs e)
		{
			TimeSpan idleTime = IdleInputTracker.IdleTime;
			if(!this.triggered && (idleTime.TotalMilliseconds >= triggerTime))
			{
				this.triggered = true;
				this.OnInputIdle(EventArgs.Empty);
			}
			else if(this.triggered && (idleTime.TotalMilliseconds < triggerTime))
			{
				this.triggered = false;
				this.OnInputActive(EventArgs.Empty);
			}
		}

		private struct LASTINPUTINFO
		{
			public int size;
			public uint ticks;
		}

		[System.Runtime.InteropServices.DllImport("user32")]
		private static extern bool GetLastInputInfo([System.Runtime.InteropServices.In,System.Runtime.InteropServices.Out] ref LASTINPUTINFO lii);

		[System.Runtime.InteropServices.DllImport("kernel32")]
		private static extern uint GetTickCount();

		public static TimeSpan IdleTime
		{
			get
			{
				LASTINPUTINFO lii = new LASTINPUTINFO();
				lii.size = System.Runtime.InteropServices.Marshal.SizeOf(typeof(LASTINPUTINFO));
				GetLastInputInfo(ref lii);
				uint tc = GetTickCount();
				if(tc >= lii.ticks)
				{
					return TimeSpan.FromMilliseconds(tc - lii.ticks);
				}
				else
				{
					return TimeSpan.FromMilliseconds(ulong.MaxValue - lii.ticks + tc);
				}
			}
		}


		[Description("Time, in milliseconds, before triggering the InputIdle event."), DefaultValue(5 * 60 * 1000)]
		public int Interval
		{
			get
			{
				return this.triggerTime;
			}
			set
			{
				this.triggerTime = value;
			}
		}

		[Description("Occurs when there has been no user input for the duration given by the TiggerTime property.")]
		public event EventHandler InputIdle;

		protected virtual void OnInputIdle(EventArgs e)
		{
			if(null != this.InputIdle)
			{
				this.InputIdle(this, e);
			}
		}

		public event EventHandler InputActive;

		protected virtual void OnInputActive(EventArgs e)
		{
			if(null != this.InputActive)
			{
				this.InputActive(this, e);
			}
		}

		[Description("Gets or sets a value that determines if this tracker is active."),DefaultValue(true)]
		public bool Enabled
		{
			get
			{
				return this.timer1.Enabled;
			}
			set
			{
				this.timer1.Enabled = value;
			}
		}
	}
}
