using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using GroupLab.Networking;

namespace PresenceItem
{
	/// <summary>
	/// Summary description for InviteForm.
	/// </summary>
	public class InviteForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button okBtn;
		private System.Windows.Forms.ListBox placeListBox;
		private System.Windows.Forms.Button cancelBtn;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		GroupLab.Networking.SharedDictionary sd;
		string ownerKey;
		string ownerName;
		private System.Windows.Forms.Label noPlaceLbl;
		string currentUserKey;

		public InviteForm(GroupLab.Networking.SharedDictionary sd, string placeKey, string ownerKey, string currentUserKey)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.sd = sd;
			this.ownerKey = ownerKey;
			this.ownerName = this.sd[ownerKey] as string;
			this.Text = "Invite " + this.ownerName;
			this.currentUserKey = currentUserKey;

			// do the colours
			GroupLab.Networking.Map seedcolour = this.sd[placeKey+"/colour"] as Map;
			ColourGenerator.ColourGenerator colour = new ColourGenerator.ColourGenerator(System.Drawing.Color.FromArgb((int)seedcolour["red"], (int)seedcolour["green"], (int)seedcolour["blue"]));
			this.BackColor = colour.BaseColour;
			this.okBtn.ForeColor = colour.BaseColour;
			this.cancelBtn.ForeColor = colour.BaseColour;
			this.cancelBtn.BackColor = colour.DarkColour;
			this.okBtn.BackColor = colour.DarkColour;

			this.noPlaceLbl.Text = this.ownerName + " is already a member of all the places that you are in.";
			this.noPlaceLbl.Visible = false;
			this.PopulateListBox();
		}

		private void PopulateListBox()
		{
			// populate the list box with all places that the current user
			// is subscribed to but the owner isn't
			ArrayList currentPlaces = this.PlacesForUser(this.currentUserKey);
			ArrayList ownerPlaces = this.PlacesForUser(this.ownerKey);

			foreach(string placeKey in ownerPlaces)
			{
				if(currentPlaces.Contains(placeKey))
				{
					currentPlaces.Remove(placeKey);
				}
			}
			if(currentPlaces.Count == 0)
			{
				this.noPlaceLbl.Visible = true;
				this.placeListBox.Visible = false;
			}
			foreach(string placeKey in currentPlaces)
			{
				string placeName = this.sd[placeKey] as string;
				MiniPlace p = new MiniPlace(placeKey, placeName);
				this.placeListBox.Items.Add(p);
			}
		}

		private ArrayList PlacesForUser(string userKey)
		{
			ArrayList placeKeys = new ArrayList();
			foreach(GroupLab.Networking.SharedDictionary.Entry e in (this.sd["/places/?/member/?"] as IEnumerable))
			{
				if((string)(e.Value) == userKey)
				{
					int slashIndex = e.Path.LastIndexOf("/member");
					string groupKey = e.Path.Substring(0,slashIndex);

					placeKeys.Add(groupKey);
				}
			}
			/* include all the places that the user is invited to as well */
			foreach(GroupLab.Networking.SharedDictionary.Entry e in (this.sd["/places/?/invited/?"] as IEnumerable))
			{
				GroupLab.Networking.Map m = e.Value as Map;
				if(null != m && m.Contains("invitee") && (m["invitee"] as string) == userKey)
				{
					int slashIndex = e.Path.LastIndexOf("/invited");
					string groupKey = e.Path.Substring(0,slashIndex);

					placeKeys.Add(groupKey);
				}
			}
			return placeKeys;
		}
		

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.placeListBox = new System.Windows.Forms.ListBox();
			this.okBtn = new System.Windows.Forms.Button();
			this.cancelBtn = new System.Windows.Forms.Button();
			this.noPlaceLbl = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// placeListBox
			// 
			this.placeListBox.Location = new System.Drawing.Point(8, 8);
			this.placeListBox.Name = "placeListBox";
			this.placeListBox.Size = new System.Drawing.Size(160, 134);
			this.placeListBox.TabIndex = 0;
			// 
			// okBtn
			// 
			this.okBtn.BackColor = System.Drawing.Color.MediumBlue;
			this.okBtn.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
			this.okBtn.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.okBtn.ForeColor = System.Drawing.Color.SkyBlue;
			this.okBtn.Location = new System.Drawing.Point(8, 152);
			this.okBtn.Name = "okBtn";
			this.okBtn.TabIndex = 1;
			this.okBtn.Text = "OK";
			this.okBtn.Click += new System.EventHandler(this.okBtn_Click);
			// 
			// cancelBtn
			// 
			this.cancelBtn.BackColor = System.Drawing.Color.MediumBlue;
			this.cancelBtn.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
			this.cancelBtn.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.cancelBtn.ForeColor = System.Drawing.Color.SkyBlue;
			this.cancelBtn.Location = new System.Drawing.Point(96, 152);
			this.cancelBtn.Name = "cancelBtn";
			this.cancelBtn.TabIndex = 2;
			this.cancelBtn.Text = "Cancel";
			this.cancelBtn.Click += new System.EventHandler(this.cancelBtn_Click);
			// 
			// noPlaceLbl
			// 
			this.noPlaceLbl.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.noPlaceLbl.Location = new System.Drawing.Point(8, 8);
			this.noPlaceLbl.Name = "noPlaceLbl";
			this.noPlaceLbl.Size = new System.Drawing.Size(160, 134);
			this.noPlaceLbl.TabIndex = 3;
			this.noPlaceLbl.Text = "No Places Available";
			this.noPlaceLbl.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// InviteForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.Color.Azure;
			this.ClientSize = new System.Drawing.Size(176, 181);
			this.Controls.Add(this.noPlaceLbl);
			this.Controls.Add(this.cancelBtn);
			this.Controls.Add(this.okBtn);
			this.Controls.Add(this.placeListBox);
			this.Name = "InviteForm";
			this.Text = "Invite";
			this.ResumeLayout(false);

		}
		#endregion

		private void okBtn_Click(object sender, System.EventArgs e)
		{
			System.Collections.ICollection selected = this.placeListBox.SelectedItems;
			foreach(MiniPlace p in selected)
			{
				GroupLab.Networking.Map invite = new Map();
				invite["type"] = "invitation";
				invite["owner"] = this.currentUserKey;
				invite["place"] = p.Key;
				invite["inviter"] = this.currentUserKey;
				invite["invitee"] = this.ownerKey;
				this.sd[p.Key+"/invited/"+Guid.NewGuid().ToString()] = invite;
			}
			this.Close();
		}

		private void cancelBtn_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

	}

	public class MiniPlace
	{
		string key;
		string name;

		public MiniPlace(string placeKey, string name)
		{
			this.key = placeKey;
			this.name = name;
		}

		public override string ToString()
		{
			return this.name;
		}

		public string Key
		{
			get
			{
				return this.key;
			}
		}
	}
}
