using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace PresenceItem
{
	/// <summary>
	/// Summary description for MasterTransient.
	/// </summary>
	public class MasterTransient : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.ComboBox statusCombo;
		private System.Windows.Forms.RichTextBox messageBox;
		private System.Windows.Forms.Panel picturePanel;
		private System.Windows.Forms.PictureBox pictureBox;
		private System.Windows.Forms.PictureBox videoBox;
		private System.Windows.Forms.Label videoLbl;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Label pictureLbl;
		private System.Windows.Forms.ToolTip pictureToolTip;
		private System.Windows.Forms.ToolTip videoToolTip;

		PresenceItem presence;

		public MasterTransient(PresenceItem presence)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			this.presence = presence;

			//this.changePictureBtn.BackColor = this.presence.Colour.DarkColour;
			//this.changePictureBtn.ForeColor = this.presence.Colour.BaseColour;

			this.RefreshView();

			this.presence.ImageChanged += new EventHandler(this.presence_ImageChanged);
			this.presence.PictureChanged += new EventHandler(this.presence_PictureChanged);
			this.presence.NimbusChanged += new EventHandler(this.presence_NimbusChanged);
			this.presence.StatusChanged += new EventHandler(this.presence_StatusChanged);

			this.Paint += new PaintEventHandler(this.MasterTransient_Paint);
		}

		private void RefreshView()
		{
			if(this.presence.Nimbus >= this.presence.VideoAwareness)
			{
				this.videoBox.Enabled = true;
				this.videoLbl.Visible = false;
				Image i = this.videoBox.Image;
				this.videoBox.Image = this.presence.GetImage(this.videoBox.Size);
				if(null != i)
				{
					i.Dispose();
				}
			}
			else
			{
				this.videoBox.Enabled = false;
				this.videoLbl.Visible = true;
			}
			if(this.presence.Nimbus >= this.presence.PictureAwareness)
			{
				this.pictureBox.Enabled = true;
				this.pictureLbl.Visible = false;
				//Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetPicture(this.pictureBox.Size);
//				if(null != i)
//				{
//					i.Dispose();
//				}
			}
			else
			{
				this.pictureBox.Enabled = false;
				this.pictureLbl.Visible = true;
			}
			if(this.presence.Nimbus >= this.presence.MessageAwareness)
			{
				this.messageBox.Enabled = true;
				this.messageBox.Text = this.presence.Message;
			}
			else
			{
				this.messageBox.Enabled = false;
				this.messageBox.Text = this.presence.Message;
			}
			if(this.presence.Nimbus >= this.presence.StatusAwareness)
			{
				this.statusCombo.Enabled = true;
				this.statusCombo.Text = this.presence.Status;
			}
			else
			{
				this.statusCombo.Text = "No Status";
				this.statusCombo.Enabled = false;
			}
		}


		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			this.presence.ImageChanged -= new EventHandler(this.presence_ImageChanged);
			this.presence.PictureChanged -= new EventHandler(this.presence_PictureChanged);
			this.presence.NimbusChanged -= new EventHandler(this.presence_NimbusChanged);
			this.presence.StatusChanged -= new EventHandler(this.presence_StatusChanged);

			this.Paint -= new PaintEventHandler(this.MasterTransient_Paint);

			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.statusCombo = new System.Windows.Forms.ComboBox();
			this.messageBox = new System.Windows.Forms.RichTextBox();
			this.picturePanel = new System.Windows.Forms.Panel();
			this.pictureLbl = new System.Windows.Forms.Label();
			this.pictureBox = new System.Windows.Forms.PictureBox();
			this.videoBox = new System.Windows.Forms.PictureBox();
			this.videoLbl = new System.Windows.Forms.Label();
			this.pictureToolTip = new System.Windows.Forms.ToolTip(this.components);
			this.videoToolTip = new System.Windows.Forms.ToolTip(this.components);
			this.picturePanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// statusCombo
			// 
			this.statusCombo.Dock = System.Windows.Forms.DockStyle.Top;
			this.statusCombo.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.statusCombo.Items.AddRange(new object[] {
															 "Online",
															 "Away",
															 "Busy",
															 "Be Right Back",
															 "Out to Lunch",
															 "On the Phone"});
			this.statusCombo.Location = new System.Drawing.Point(0, 0);
			this.statusCombo.Name = "statusCombo";
			this.statusCombo.Size = new System.Drawing.Size(150, 19);
			this.statusCombo.TabIndex = 0;
			this.statusCombo.Text = "Online";
			this.statusCombo.KeyUp += new System.Windows.Forms.KeyEventHandler(this.statusCombo_KeyUp);
			this.statusCombo.SelectedIndexChanged += new System.EventHandler(this.statusCombo_SelectedIndexChanged);
			// 
			// messageBox
			// 
			this.messageBox.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.messageBox.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.messageBox.Location = new System.Drawing.Point(0, 114);
			this.messageBox.Name = "messageBox";
			this.messageBox.Size = new System.Drawing.Size(150, 30);
			this.messageBox.TabIndex = 1;
			this.messageBox.Text = "";
			this.messageBox.TextChanged += new System.EventHandler(this.messageBox_TextChanged);
			// 
			// picturePanel
			// 
			this.picturePanel.Controls.Add(this.pictureLbl);
			this.picturePanel.Controls.Add(this.pictureBox);
			this.picturePanel.Dock = System.Windows.Forms.DockStyle.Left;
			this.picturePanel.Location = new System.Drawing.Point(0, 19);
			this.picturePanel.Name = "picturePanel";
			this.picturePanel.Size = new System.Drawing.Size(75, 95);
			this.picturePanel.TabIndex = 2;
			// 
			// pictureLbl
			// 
			this.pictureLbl.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.pictureLbl.Location = new System.Drawing.Point(8, 32);
			this.pictureLbl.Name = "pictureLbl";
			this.pictureLbl.Size = new System.Drawing.Size(64, 23);
			this.pictureLbl.TabIndex = 1;
			this.pictureLbl.Text = "No Picture";
			this.pictureLbl.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// pictureBox
			// 
			this.pictureBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.pictureBox.Location = new System.Drawing.Point(0, 0);
			this.pictureBox.Name = "pictureBox";
			this.pictureBox.Size = new System.Drawing.Size(75, 95);
			this.pictureBox.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage;
			this.pictureBox.TabIndex = 0;
			this.pictureBox.TabStop = false;
			this.pictureBox.Click += new System.EventHandler(this.pictureBox_Click);
			// 
			// videoBox
			// 
			this.videoBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.videoBox.Location = new System.Drawing.Point(75, 19);
			this.videoBox.Name = "videoBox";
			this.videoBox.Size = new System.Drawing.Size(75, 95);
			this.videoBox.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage;
			this.videoBox.TabIndex = 3;
			this.videoBox.TabStop = false;
			this.videoBox.Click += new System.EventHandler(this.videoBox_Click);
			// 
			// videoLbl
			// 
			this.videoLbl.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.videoLbl.Location = new System.Drawing.Point(80, 53);
			this.videoLbl.Name = "videoLbl";
			this.videoLbl.Size = new System.Drawing.Size(64, 19);
			this.videoLbl.TabIndex = 4;
			this.videoLbl.Text = "No Video";
			this.videoLbl.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// pictureToolTip
			// 
			this.pictureToolTip.AutomaticDelay = 100;
			// 
			// videoToolTip
			// 
			this.videoToolTip.AutomaticDelay = 100;
			this.videoToolTip.AutoPopDelay = 5000;
			this.videoToolTip.InitialDelay = 100;
			this.videoToolTip.ReshowDelay = 20;
			// 
			// MasterTransient
			// 
			this.BackColor = System.Drawing.Color.SkyBlue;
			this.Controls.Add(this.videoLbl);
			this.Controls.Add(this.videoBox);
			this.Controls.Add(this.picturePanel);
			this.Controls.Add(this.messageBox);
			this.Controls.Add(this.statusCombo);
			this.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Name = "MasterTransient";
			this.Size = new System.Drawing.Size(150, 144);
			this.picturePanel.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void presence_ImageChanged(object sender, EventArgs e)
		{
			if(this.presence.Nimbus >= this.presence.VideoAwareness)
			{
				Image i = this.videoBox.Image;
				this.videoBox.Image = this.presence.GetImage(this.videoBox.Size);
				if(null != i)
				{
					i.Dispose();
				}
			}
		}

		private void presence_PictureChanged(object sender, EventArgs e)
		{
			if(this.presence.Nimbus >= this.presence.PictureAwareness)
			{
				//Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetPicture(this.pictureBox.Size);
//				if(null != i)
//				{
//					i.Dispose();
//				}
			}
		}

		private void presence_NimbusChanged(object sender, EventArgs e)
		{
			this.RefreshView();
		}

		private void statusCombo_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			this.presence.Status = this.statusCombo.SelectedItem.ToString();
		}

		private void statusCombo_KeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			if(e.KeyCode == System.Windows.Forms.Keys.Enter)
			{
				this.presence.Status = this.statusCombo.Text;
			}
		}

		private void messageBox_TextChanged(object sender, System.EventArgs e)
		{
			this.presence.Message = this.messageBox.Text;
		}

		private void presence_StatusChanged(object sender, EventArgs e)
		{
			if(this.presence.Status == "Online")
			{
				this.pictureBox.BackColor = this.presence.Colour.PresenceColour;
				this.pictureLbl.BackColor = this.presence.Colour.PresenceColour;
				this.videoBox.BackColor = this.presence.Colour.PresenceColour;
				this.videoLbl.BackColor = this.presence.Colour.PresenceColour;
			}
			else
			{
				this.pictureBox.BackColor = this.presence.Colour.AwayColour;
				this.pictureLbl.BackColor = this.presence.Colour.AwayColour;
				this.videoBox.BackColor = this.presence.Colour.AwayColour;
				this.videoLbl.BackColor = this.presence.Colour.AwayColour;
			}
		}

		private void videoBox_Click(object sender, System.EventArgs e)
		{
			if(this.presence.Awareness >= this.presence.VideoAwareness)
			{
				this.presence.UpdateCamera();
			}
		}

		private void MasterTransient_Paint(object sender, PaintEventArgs e)
		{
			this.Focus();
			this.messageBox.Focus();
			this.messageBox.SelectAll();
			this.pictureToolTip.SetToolTip(this.pictureBox, "Click to Change");
			this.videoToolTip.SetToolTip(this.videoBox, "Click to Update");
		}

		private void pictureBox_Click(object sender, System.EventArgs e)
		{
			// show a file load dialog
			System.Windows.Forms.OpenFileDialog fd = new OpenFileDialog();
			if(System.Windows.Forms.DialogResult.OK == fd.ShowDialog())
			{
				try
				{
					this.presence.SetPicture(fd.FileName);
					// For logging purpose - [NATA] - 29.07.2005
					this.presence.logPictureChange(fd.FileName);
				}
				catch
				{
					string msg = "There was an error trying to load your file.\n\nCurrently only JPEG, PNG, and BMP files are supported. Check that your file is one of these formats.";
					System.Windows.Forms.MessageBox.Show(msg, "Error loading picture file.", System.Windows.Forms.MessageBoxButtons.OK);
				}
			}		
		}
	}
}
