using System;
using SideItem;
using Collabrary;
using System.Windows;
using System.Drawing;
using System.Diagnostics;
using GroupLab.Networking;
using System.Collections;
using System.Windows.Forms;
using System.Reflection;

namespace PresenceItem
{
	/// <summary>
	/// Simple video presence item for the sidebar.
	/// 
	/// Shows a video snapshot for the current user.
	/// </summary>
	/// 
	[SideItem.ItemName("Presence")]
	public class PresenceItem : SideItem.IItem
	{
		// determines whether we display awareness of others' view on us
		const bool displayAwareness = false;

		string path;
		GroupLab.Networking.SharedDictionary sd;
		bool master;
		string currentUserKey;
		string placeKey;

		PresenceTile tile;
		ArrayList transients;
		System.Windows.Forms.Form separate;

		private string nimbusPath;
		private GroupLab.Networking.Subscription nimbusSub;
		private string focusPath;
		private GroupLab.Networking.Subscription focusSub;

		// other awareness
		private string currentUserNimbusPath;
		private GroupLab.Networking.Subscription currentUserNimbusSub;
		private string ownerFocusPath;
		private GroupLab.Networking.Subscription ownerFocusSub;

		private bool autoAway;
		private Collabrary.Photo defaultPicture;

		// message stuff
		private string messagePath;
		private GroupLab.Networking.Subscription messageSub;

		// Video stuff
		private Collabrary.Camera bigCamera;
		private GroupLab.Networking.Subscription bigImageSub;
		private GroupLab.Networking.Subscription bigImageRequestSub;
		private string bigImagePath;
		private string bigImageRequestPath;
		private GroupLab.Networking.Subscription allowBigVideoSub;
		private string allowBigVideoPath;

		private Collabrary.Camera camera;
		private GroupLab.Networking.Subscription imageSub;
		private const int imageHeight = 144;
		private const int imageWidth = 172;
		private Image image;
		private string imagePath;

		// status stuff
		private IdleInputTracker idle;
		private GroupLab.Networking.Subscription activitySub;
		private string statusPath;

		private int focus;
		private int nimbus;

		// picture stuff
		GroupLab.Networking.Subscription pictureSub;
		private string picturePath;

		Collabrary.Speaker spkr;
		GroupLab.Networking.Subscription soundSub;
		private string soundPath;

		ColourGenerator.ColourGenerator colour;

		// testing
		System.Windows.Forms.Timer testTmr;
		private int counter;

		public PresenceItem()
		{ 
			// testing
//			this.counter = 0;
//			this.testTmr = new Timer();
//			this.testTmr.Interval = 500;
//			this.testTmr.Tick += new EventHandler(testTmr_Tick);
//			this.testTmr.Start();

			this.focus = 100;
			this.nimbus = 100;

			this.transients = new ArrayList();

			this.autoAway = false;

			string location = Assembly.GetExecutingAssembly().Location;
			System.IO.DirectoryInfo dir = System.IO.Directory.GetParent(location);
			string pictureFile = System.IO.Path.Combine(dir.FullName, "./default.jpg");
			this.defaultPicture = new Collabrary.PhotoClass();
			this.defaultPicture.Load(pictureFile);

			this.tile = new PresenceTile(this);
		}

		#region getters and setters

		public int StatusAwareness
		{
			get
			{
				return 0;
			}
		}
		public int NameAwareness
		{
			get
			{
				return 10;
			}
		}
		public int MessageAwareness
		{
			get
			{
				return 20;
			}
		}
		public int PictureAwareness
		{
			get
			{
				return 40;
			}
		}
		public int VideoAwareness
		{
			get
			{
				return 90;
			}
		}

		public ColourGenerator.ColourGenerator Colour
		{
			get
			{
				return this.colour;
			}
		}
		public int NumVideoRequests
		{
			get
			{
				int numRequests = 0;
				if(null != this.sd && this.sd.Contains(this.bigImageRequestPath))
				{
					try
					{
						numRequests = (int)this.sd[this.bigImageRequestPath];
					}
					catch
					{
						numRequests = 0;
					}
				}
				return numRequests;
			}
		}
		public bool AllowBigVideo
		{
			get
			{
				string allow = "no";
				if(null != this.sd && this.sd.Contains(this.allowBigVideoPath))
				{
					allow = this.sd[this.allowBigVideoPath] as string;
				}
				return (allow == "yes");
			}
			set
			{
				string allow;
				if(value)
				{
					allow = "yes";
				}
				else
				{
					allow = "no";
				}
				this.sd[this.allowBigVideoPath] = allow;
			}
		}

		public string Message
		{
			get
			{
				if(null != this.sd && this.sd.Contains(this.messagePath))
				{
					return this.sd[this.messagePath] as string;
				}
				else
				{
					return string.Empty;
				}
			}
			set
			{
				if(this.Master)
				{
					this.sd[this.messagePath] = value;
				}
			}
		}

		public string PlaceName
		{
			get
			{
				if(null != this.sd && null != this.path)
				{
					Map m = this.sd[this.path] as Map;
					string placeKey = m["place"] as string;
					return this.sd[placeKey] as string;
				}
				else
				{
					return string.Empty;
				}
			}
		}
		public Image GetBigImage(Size size)
		{
			Image i;
			try
			{
				Photo p;
				Collabrary.Buffer v = (Collabrary.Buffer)this.sd[this.bigImagePath];
				JPEG j = new JPEGClass();
				p = (Photo)j.Decompress(v);
				this.ScalePhoto(size, p);
				i = Bitmap.FromHbitmap(p.Hbitmap);
				System.Runtime.InteropServices.Marshal.ReleaseComObject(p);
			} 
			catch(Exception) 
			{
				i = this.GetImage(size);
			}
			return i;
		}
		public void RequestBigVideo()
		{
			if(null != this.sd)
			{
				int numRequests = 0;
				if(this.sd.Contains(this.bigImageRequestPath))
				{
					numRequests = (int)this.sd[this.bigImageRequestPath];
				}
				numRequests += 1;
				this.sd[this.bigImageRequestPath] = numRequests;
			}
		}
		public void ReleaseBigVideo()
		{
			if(null != this.sd)
			{
				int numRequests = 0;
				if(this.sd.Contains(this.bigImageRequestPath))
				{
					numRequests = (int)this.sd[this.bigImageRequestPath];
					numRequests -= 1;
				}
				this.sd[this.bigImageRequestPath] = numRequests;
			}
		}

		public Image GetImage(Size size)
		{
			Photo p;
			try
			{
				Collabrary.Buffer v = (Collabrary.Buffer)this.sd[this.imagePath];
				JPEG j = new JPEGClass();
				p = (Photo)j.Decompress(v);
			} 
			catch(Exception) 
			{
				p = this.defaultPicture.Copy(0,0,0,0,Collabrary.PhotoCopyStyle.Pixels) as Photo;
			}
			this.ScalePhoto(size, p);
			Image i = Bitmap.FromHbitmap(p.Hbitmap);
			System.Runtime.InteropServices.Marshal.ReleaseComObject(p);
			return i;
		}

		/// <summary>
		/// Image returned is smaller than size and retains its proportions.
		/// </summary>
		/// <param name="size">size of the area to contain the image</param>
		/// <returns>an image smaller than size</returns>
		public Image GetPicture(Size size)
		{
			Photo p;
			try
			{
				Collabrary.Buffer v = this.sd[this.picturePath] as Collabrary.Buffer;
				JPEG j = new JPEGClass();
				p = (Photo)j.Decompress(v);
			} 
			catch(Exception) 
			{
				p = this.defaultPicture.Copy(0,0,0,0,Collabrary.PhotoCopyStyle.Pixels) as Photo;
			}
			this.ScalePhoto(size, p);
			Image i = Bitmap.FromHbitmap(p.Hbitmap);
			System.Runtime.InteropServices.Marshal.ReleaseComObject(p);
			return i;
		}

		public void ScalePhoto(Size size, Collabrary.IPhoto p)
		{
			float photoRatio = (float)p.Height/(float)p.Width;
			// try to make the image match the width
			int newHeight = (int)(size.Width*photoRatio);
			if(newHeight < size.Height)
			{
				// it worked
				p.Resize(size.Width, newHeight, true);
			}
			else
			{
				// have to scale to the size.height instead
				p.Resize((int)((float)size.Height*(1/photoRatio)), size.Height, true);
			}
		}

		public void SetPicture(string fileName)
		{
			Collabrary.Photo p = new Collabrary.PhotoClass();
			//p.SetSize(800, 600);
			p.Load(fileName);
			this.ScalePhoto(new Size(152,132), p);

			Collabrary.JPEG j = new Collabrary.JPEG(); 
			Collabrary.Buffer data = j.Compress(p);

			this.sd[this.picturePath] = data;

			System.Runtime.InteropServices.Marshal.ReleaseComObject(p);
			System.Runtime.InteropServices.Marshal.ReleaseComObject(j);
		}

		public string Status
		{
			get
			{
				if(null != this.sd && this.sd.Contains(this.statusPath))
				{
					return this.sd[this.statusPath] as string;
				}
				else
				{
					return "Online";
				}
			}
			set
			{
				if(this.Master)
				{
					this.sd[this.statusPath] = value;
				}
			}
		}

		public bool IsDisplayAwareness
		{
			get
			{
				return displayAwareness;
			}
		}

		// returns the owner's awareness of the current user
		public int OwnerAwareness
		{
			get
			{
				int currentUserNimbus;
				if(null != this.sd && this.sd.Contains(this.currentUserNimbusPath))
				{
					currentUserNimbus = (int)this.sd[this.currentUserNimbusPath];
				}
				else
				{
					currentUserNimbus = 100;
				}

				int ownerFocus = (int)this.sd[this.ownerFocusPath];
				if(null != this.sd && this.sd.Contains(this.ownerFocusPath))
				{
					ownerFocus = (int)this.sd[this.ownerFocusPath];
				}
				else
				{
					ownerFocus = 100;
				}

				return (int)((float)ownerFocus/100F*(float)currentUserNimbus);
			}
		}

		public int UserFocus
		{
			get
			{
				if(null != this.sd && this.sd.Contains(this.focusPath))
				{
					this.nimbus = (int)this.sd[this.focusPath];
				}
				return this.focus;
			}
			set
			{
				if(null != this.focusPath)
				{
					this.sd[this.focusPath] = value;
				}
				else
				{
					this.focus = value;
				}
			}
		}

		public int Awareness
		{
			get
			{
				return (int)((float)this.UserFocus/100.0*(float)this.Nimbus);
			}
		}

		public int Nimbus
		{
			get
			{
				if(null != this.sd && this.sd.Contains(this.nimbusPath))
				{
					this.nimbus = (int)this.sd[this.nimbusPath];
				}
				return this.nimbus;
			}
			set
			{
				if(this.Master)
				{
					this.sd[this.nimbusPath] = value;
				}
			}
		}

		public bool Master
		{
			get
			{
				return this.master;
			}
		}

		public string OwnerKey
		{
			get
			{
				if(null != this.sd && this.sd.Contains(this.path))
				{
					GroupLab.Networking.Map info = this.sd[this.path] as GroupLab.Networking.Map;
					return info["owner"] as string;
				}
				else
				{
					return string.Empty;
				}
			}
		}

		public string CurrentUserKey
		{
			get
			{
				return this.currentUserKey;
			}
		}

		public string OwnerName
		{
			get
			{
				if(null != this.sd && this.sd.Contains(this.OwnerKey))
				{
					return this.sd[this.OwnerKey] as string;
				}
				else
				{
					return "<name>";
				}
			}
		}
		#endregion

		#region IItem Members

		public void StartItem(GroupLab.Networking.SharedDictionary sd, string path, bool master, string currentUserKey)
		{
			this.sd = sd;
			this.path = path;
			this.master = master;		
			this.currentUserKey = currentUserKey;

			this.placeKey = (this.sd[this.path] as Map)["place"] as String;
			string ownerKey = (this.sd[this.path] as Map)["owner"] as string;

			GroupLab.Networking.Map seedcolour = this.sd[this.placeKey+"/colour"] as Map;
			this.colour = new ColourGenerator.ColourGenerator(System.Drawing.Color.FromArgb((int)seedcolour["red"], (int)seedcolour["green"], (int)seedcolour["blue"]));
			this.OnStatusChanged(new EventArgs());

			this.separate = null;

			this.nimbusPath = placeKey+"/"+ownerKey+"/nimbus";
			this.nimbusSub = new GroupLab.Networking.Subscription();
			this.nimbusSub.BeginInit();
			this.nimbusSub.Pattern = this.nimbusPath;
			this.nimbusSub.Dictionary = this.sd;
			this.nimbusSub.Enabled = true;
			//this.nimbusSub.SubscriptionChanged += new SubscriptionChangedEventHandler(this.nimbusSub_SubscriptionChanged);
			this.nimbusSub.Notified += new SubscriptionEventHandler(this.nimbusSub_Notified);
			this.nimbusSub.EndInit();

			this.focusPath = placeKey+this.OwnerKey+this.CurrentUserKey+"/focus";
			this.focusSub = new GroupLab.Networking.Subscription();
			this.focusSub.BeginInit();
			this.focusSub.Pattern = this.focusPath;
			this.focusSub.Dictionary = this.sd;
			this.focusSub.Enabled = true;
			this.focusSub.Notified += new SubscriptionEventHandler(this.focusSub_Notified);
			this.focusSub.EndInit();

			if(!this.master)
			{
				// track the owner's awareness of the current user
				this.currentUserNimbusPath = this.placeKey+this.CurrentUserKey+"/nimbus";
				this.currentUserNimbusSub = new GroupLab.Networking.Subscription();
				this.currentUserNimbusSub.BeginInit();
				this.currentUserNimbusSub.Pattern = this.currentUserNimbusPath;
				this.currentUserNimbusSub.Dictionary = this.sd;
				this.currentUserNimbusSub.Enabled = true;
				this.currentUserNimbusSub.Notified += new SubscriptionEventHandler(this.ownerAwareness_Notified);
				this.currentUserNimbusSub.EndInit();

				this.ownerFocusPath = this.placeKey+this.CurrentUserKey+this.OwnerKey+"/focus";
				this.ownerFocusSub = new GroupLab.Networking.Subscription();
				this.ownerFocusSub.BeginInit();
				this.ownerFocusSub.Pattern = this.ownerFocusPath;
				this.ownerFocusSub.Dictionary = this.sd;
				this.ownerFocusSub.Enabled = true;
				this.ownerFocusSub.Notified += new SubscriptionEventHandler(this.ownerAwareness_Notified);
				this.ownerFocusSub.EndInit();
			}

			// message stuff
			this.messagePath = this.path+"/message";
			this.messageSub = new GroupLab.Networking.Subscription();
			this.messageSub.BeginInit();
			this.messageSub.Dictionary = this.sd;
			this.messageSub.Pattern = this.messagePath;
			this.messageSub.Enabled = true;
			this.messageSub.Notified += new SubscriptionEventHandler(this.messageSub_Notified);
			this.messageSub.EndInit();

			// subscribe to images that are added
			this.imagePath = this.path+"/snapshot";
			this.imageSub = new GroupLab.Networking.Subscription();
			this.imageSub.BeginInit();
			this.imageSub.Pattern = this.imagePath;
			this.imageSub.Enabled = true;
			this.imageSub.Dictionary = this.sd;
			this.imageSub.Notified += new SubscriptionEventHandler(this.imageSub_Notified);
			this.imageSub.EndInit();

			// status stuff
			this.statusPath = this.path+"/status";
			this.activitySub = new GroupLab.Networking.Subscription();
			this.activitySub.BeginInit();
			this.activitySub.Dictionary = this.sd;
			this.activitySub.Pattern = this.statusPath;
			this.activitySub.Notified += new SubscriptionEventHandler(this.activitySub_Notified);
			this.activitySub.EndInit();

			// big video stuff
			this.allowBigVideoPath = placeKey+"/"+ownerKey+"/allowBigVideo";

			this.allowBigVideoSub = new GroupLab.Networking.Subscription();
			this.allowBigVideoSub.BeginInit();
			this.allowBigVideoSub.Pattern = this.allowBigVideoPath;
			this.allowBigVideoSub.Dictionary = this.sd;
			this.allowBigVideoSub.Notified += new SubscriptionEventHandler(this.allowBigVideoSub_Notified);
			this.allowBigVideoSub.EndInit();

			this.bigImagePath = this.path+"/bigImage";
			this.bigImageSub = new GroupLab.Networking.Subscription();
			this.bigImageSub.BeginInit();
			this.bigImageSub.Dictionary = this.sd;
			this.bigImageSub.Pattern = this.bigImagePath;
			this.bigImageSub.Notified += new SubscriptionEventHandler(this.bigImageSub_Notified);
			this.bigImageSub.EndInit();
			this.bigImageRequestPath = this.path+"/bigImageRequest";

			if(this.master)
			{				
				this.StartVideo();

				this.bigImageRequestSub = new GroupLab.Networking.Subscription();
				this.bigImageRequestSub.BeginInit();
				this.bigImageRequestSub.Pattern = this.bigImageRequestPath;
				this.bigImageRequestSub.Dictionary = this.sd;
				this.bigImageRequestSub.Notified += new SubscriptionEventHandler(this.bigImageRequestSub_Notified);
				this.bigImageRequestSub.EndInit();

				this.idle = new IdleInputTracker(1000, 5*60*1000);
				this.idle.InputIdle += new EventHandler(this.idle_InputIdle);
				this.idle.InputActive += new EventHandler(this.idle_InputActive);
			}

			// picture stuff
			// put the picture with the place data

			this.picturePath = placeKey+"/"+ownerKey+"/picture";
			this.pictureSub = new GroupLab.Networking.Subscription();
			this.pictureSub.BeginInit();
			this.pictureSub.Dictionary = this.sd;
			this.pictureSub.Pattern = this.picturePath;
			this.pictureSub.Notified += new SubscriptionEventHandler(this.pictureSub_Notified);
			this.pictureSub.EndInit();

			this.OnNameChanged(new EventArgs());

			this.soundPath = this.path+"/sound";
			if(this.master)
			{
				// set up the sound stuff
				this.spkr = new Collabrary.SpeakerClass();				
				this.spkr.Buffering = 1.0F;
				this.spkr.ResyncTimeout = 1;

				this.soundSub = new GroupLab.Networking.Subscription();
				this.soundSub.BeginInit();
				this.soundSub.Pattern = this.soundPath;
				this.soundSub.Dictionary = this.sd;
				this.soundSub.Enabled = true;
				this.soundSub.Notified += new SubscriptionEventHandler(this.soundSub_Notified);
				this.soundSub.EndInit();
			}
			this.tile.SetupTile();

			this.OnNimbusChanged(new EventArgs());
			this.OnAllowBigVideoChanged(new EventArgs());
			this.OnPictureChanged(new EventArgs());
		}

		public void SeparateItem()
		{
			if(this.separate == null)
			{
				if(this.master)
				{
					this.separate = new MasterPresenceFull(this);
				}
				else
				{
					this.separate = new SlavePresenceFull(this);
				}

				this.separate.Closed += new EventHandler(this.separate_Closed);
				this.separate.Show();
			}
		}

		public System.Windows.Forms.Control GetTransientFocus(Size size)
		{
			Control c;
			if(this.Master)
			{
				c = new MasterTransient(this);
			}
			else
			{
				c = new SlaveTransient(this);
			}
			c.Size = size;
			c.Disposed += new EventHandler(this.t_Disposed);
			this.transients.Add(c);
			this.OnStatusChanged(new EventArgs());
			return c;
		}

		public System.Windows.Forms.Control GetTile(int focus)
		{
			this.UserFocus = focus;

			return this.tile;
		}

		public void StopItem()
		{
			this.nimbusSub.Notified -= new SubscriptionEventHandler(this.nimbusSub_Notified);
			this.messageSub.Notified -= new SubscriptionEventHandler(this.messageSub_Notified);
			this.imageSub.Notified -= new SubscriptionEventHandler(this.imageSub_Notified);
			this.activitySub.Notified -= new SubscriptionEventHandler(this.activitySub_Notified);
			this.bigImageSub.Notified -= new SubscriptionEventHandler(this.bigImageSub_Notified);
			this.allowBigVideoSub.Notified -= new SubscriptionEventHandler(this.allowBigVideoSub_Notified);
			this.pictureSub.Notified -= new SubscriptionEventHandler(this.pictureSub_Notified);

			if(this.master)
			{
				this.bigImageRequestSub.Notified -= new SubscriptionEventHandler(this.bigImageRequestSub_Notified);
				this.idle.InputIdle -= new EventHandler(this.idle_InputIdle);
				this.idle.InputActive -= new EventHandler(this.idle_InputActive);

				this.StopVideo();
				this.idle.Enabled = false;
			}
			if(this.tile != null)
			{
				this.tile.Dispose();
			}
			if(this.transients != null)
			{
				this.transients = null;
			}
			if(this.separate != null)
			{
				this.separate.Dispose();
			}
		}

		public event SideItem.ChangeEventHandler ItemChanged;

		#endregion

		#region subscription notifications

		private void soundSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Remove != e.Reason)
			{
				this.spkr.Play(e.Value as Collabrary.IWaveform);
			}
		}
		
		private void ownerAwareness_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Remove != e.Reason)
			{
				this.OnOwnerAwarenessChanged(new EventArgs());
			}
		}

		private void imageSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(e.Reason != GroupLab.Networking.SubscriptionNotification.Remove)
			{
				// we have a new image
				Image i = this.image;
				this.image = e.Value as Image;
				if(null != i)
				{
					i.Dispose();
				}
				this.OnImageChanged(new EventArgs());
			}
		}

		private void bigImageRequestSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Remove != e.Reason)
			{
				int numPeople = (int)e.Value;
				if(numPeople == 0)
				{
					this.StopBigVideo();
				}
				else
				{
					if(this.AllowBigVideo)
					{
						this.StartBigVideo();
					}
				}
			}
			
		}
		
		private void bigImageSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(this.AllowBigVideo)
			{
				if(GroupLab.Networking.SubscriptionNotification.Remove != e.Reason)
				{
					this.OnBigImageChanged(new EventArgs());
				}
			}
		}

		private void activitySub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Remove != e.Reason)
			{
				// new status
				this.OnStatusChanged(new EventArgs());
			}
			ChangeEventArgs ce = new SideItem.ChangeEventArgs(30);
			this.OnItemChanged(ce);
		}

		private void pictureSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Remove != e.Reason)
			{
				this.OnPictureChanged(new EventArgs());
			}
			ChangeEventArgs ce = new SideItem.ChangeEventArgs(60);
			this.OnItemChanged(ce);
		}

		private void messageSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Remove != e.Reason)
			{
				this.OnMessageChanged(new EventArgs());
			}
			ChangeEventArgs ce = new SideItem.ChangeEventArgs(40);
			this.OnItemChanged(ce);
		}

		private void nimbusSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Remove != e.Reason)
			{
				if(e.Value != null)
				{
					this.nimbus = (int)e.Value;
					this.OnNimbusChanged(new EventArgs());
					if(this.master)
					{
						if(nimbus < 90)
						{
							this.StopVideo();
						}
						else if(null == this.camera)
						{
							this.StartVideo();
						}
						else
						{
							// change the framerate of the video
							this.camera.FrameRate = 1.0F/(float)(101-nimbus);
						}
					}
				}
			}
//			ChangeEventArgs ce = new SideItem.ChangeEventArgs(2);
//			this.OnItemChanged(ce);
		}

		private void focusSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(GroupLab.Networking.SubscriptionNotification.Remove != e.Reason)
			{
				if(e.Value != null)
				{
					this.focus = (int)e.Value;
					this.OnFocusChanged(new EventArgs());
				}
			}
		}

		private void allowBigVideoSub_Notified(object sender, SubscriptionEventArgs e)
		{
			if(this.master)
			{
				if(this.AllowBigVideo && this.NumVideoRequests > 0)
				{
					this.StartBigVideo();
				}
				else if(!this.AllowBigVideo)
				{
					this.StopBigVideo();
				}
			}
			this.OnAllowBigVideoChanged(new EventArgs());
		}
		#endregion

		#region event notifiers
		public event EventHandler ImageChanged;
		protected virtual void OnImageChanged(EventArgs e)
		{
			if(null != this.ImageChanged)
			{
				this.ImageChanged(this, e);
			}
		}
		public event EventHandler OwnerAwarenessChanged;
		protected virtual void OnOwnerAwarenessChanged(EventArgs e)
		{
			if(null != this.OwnerAwarenessChanged)
			{
				this.OwnerAwarenessChanged(this, e);
			}
		}
		public event EventHandler AllowBigVideoChanged;
		protected virtual void OnAllowBigVideoChanged(EventArgs e)
		{
			if(null != this.AllowBigVideoChanged)
			{
				this.AllowBigVideoChanged(this, e);
			}
		}
		public event EventHandler BigImageChanged;
		protected virtual void OnBigImageChanged(EventArgs e)
		{
			if(null != this.BigImageChanged)
			{
				this.BigImageChanged(this, e);
			}
		}
		public event EventHandler StatusChanged;
		protected virtual void OnStatusChanged(EventArgs e)
		{
			if(null != this.StatusChanged)
			{
				this.StatusChanged(this, e);
			}
		}

		public event EventHandler PictureChanged;
		protected virtual void OnPictureChanged(EventArgs e)
		{
			if(null != this.PictureChanged)
			{
				this.PictureChanged(this, e);
			}
		}

		public event EventHandler MessageChanged;
		protected virtual void OnMessageChanged(EventArgs e)
		{
			if(null != this.MessageChanged)
			{
				this.MessageChanged(this, e);
			}
		}
		public event EventHandler NimbusChanged;
		protected virtual void OnNimbusChanged(EventArgs e)
		{
			this.tile.Nimbus = this.Nimbus;
			if(null != this.NimbusChanged)
			{
				this.NimbusChanged(this, e);
			}
		}
		public event EventHandler NameChanged;
		protected virtual void OnNameChanged(EventArgs e)
		{
			if(null != this.NameChanged)
			{
				this.NameChanged(this, e);
			}
		}
		protected virtual void OnItemChanged(SideItem.ChangeEventArgs e)
		{
			if(null != this.ItemChanged)
			{
				this.ItemChanged(this, e);
			}
		}

		public event EventHandler FocusChanged;
		protected virtual void OnFocusChanged(EventArgs e)
		{
			this.tile.UserFocus = this.UserFocus;
			if(null != this.FocusChanged)
			{
				this.FocusChanged(this, e);
			}
		}

		#endregion

		private void separate_Closed(object sender, EventArgs e)
		{
			this.ReleaseBigVideo();
			this.separate = null;
		}

		private void t_Disposed(object sender, EventArgs e)
		{
			this.transients.Remove(sender);
		}

		private void StopVideo()
		{
			if(null != this.camera)
			{
				this.camera.FrameRate = 0;
				this.camera.Captured -= new CameraEvents_CapturedEventHandler(this.camera_Captured);
				//this.camera = null;
				System.Runtime.InteropServices.Marshal.ReleaseComObject(this.camera);
				this.camera = null;
			}
		}

		private void StartVideo()
		{
			// intialise the camera
			this.camera = new Collabrary.CameraClass();
			this.camera.FrameRate = 1.0F/(float)(115-this.Nimbus);
			this.camera.Size = Collabrary.CameraSizeStyle.s160x120;

			// start capturing
			this.camera.Captured += new CameraEvents_CapturedEventHandler(this.camera_Captured);
		}

		private void idle_InputIdle(object sender, EventArgs e)
		{
			if(this.Status == "Online")
			{
				this.autoAway = true;
				this.Status = "Away";
			}
		}

		private void idle_InputActive(object sender, EventArgs e)
		{
			if(this.autoAway)
			{
				this.Status = "Online";
				this.autoAway = false;
			}
		}

		private void camera_Captured(IPhoto frame)
		{
			//this.counter = 0;
			//System.Diagnostics.Debug.WriteLine("pic captured");
			this.ScalePhoto(new Size(150,79), frame);

			Collabrary.JPEG j = new Collabrary.JPEG(); 
			Collabrary.Buffer data = j.Compress(frame);

			sd[this.imagePath] = data;
		}

		public void UpdateCamera()
		{
			this.camera_Captured(this.camera.Frame);
		}

		public void ShowInvite()
		{
			InviteForm invite = new InviteForm(this.sd, this.placeKey, this.OwnerKey, this.currentUserKey);
			invite.Show();
		}

		private void bigCamera_Captured(IPhoto Frame)
		{
			this.ScalePhoto(new Size(200,200), Frame);
			Collabrary.JPEG j = new Collabrary.JPEG(); 
			Collabrary.Buffer data = j.Compress(Frame);

			sd[this.bigImagePath] = data;
		}

		private void StartBigVideo()
		{
			this.bigCamera = new Collabrary.CameraClass();
			this.bigCamera.FrameRate = 2;
			this.bigCamera.Size = Collabrary.CameraSizeStyle.s320x240;
			this.bigCamera.Captured += new CameraEvents_CapturedEventHandler(this.bigCamera_Captured);
		}

		private void StopBigVideo()
		{
			if(null != this.bigCamera)
			{
				this.bigCamera.FrameRate = 0;
				this.bigCamera.Captured -= new CameraEvents_CapturedEventHandler(this.bigCamera_Captured);
				//this.bigCamera = null;
				System.Runtime.InteropServices.Marshal.ReleaseComObject(this.bigCamera);

				this.sd.Remove(this.bigImagePath);
				this.bigCamera = null;
			}
		}

		public void SendAudio(Collabrary.IWaveform samples)
		{
			this.sd[this.soundPath] = samples;
		}

		private void testTmr_Tick(object sender, EventArgs e)
		{
			this.counter++;
			System.Diagnostics.Debug.WriteLine(this.counter);
		}

		// For logging purpose - [NATA] - 29.07.2005
		public void logPictureChange(string pathPicture)
		{
			try
			{
				// for logging purposes
				
				string pictureKey = "/pictureFile";

				string placeId = this.placeKey;
				string path = placeId + pictureKey;
				

				GroupLab.Networking.Map map = new Map();
				
				map.Add("place", this.PlaceName);
				map.Add("type", "Presence");
				map.Add("person", this.OwnerName);
				map.Add("owner", this.OwnerKey);
				map.Add("picture", pathPicture);
				this.sd[path] = map;
			}
			catch (Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.StackTrace);
			}
		}
	}
}
