using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace PresenceItem
{
	/// <summary>
	/// Summary description for PresenceTile.
	/// </summary>
	public class PresenceTile : System.Windows.Forms.UserControl
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private PresenceItem presence;
		private System.Windows.Forms.Label messageLbl;
		private System.Windows.Forms.PictureBox pictureBox;
		private System.Windows.Forms.Panel nameStatusPanel;
		private System.Windows.Forms.Label nameLbl;

		//images
		private Image onlinePicture;
		private Image awayPicture;
		private Image busyPicture;

		private int focus;
		private System.Windows.Forms.Label ownerAwarenessLbl;
		private int nimbus;

		public PresenceTile(PresenceItem presence)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			this.focus = 100;
			this.nimbus = 100;

			this.presence = presence;

			this.presence.MessageChanged += new EventHandler(this.presence_MessageChanged);
			this.presence.ImageChanged += new EventHandler(this.presence_ImageChanged);
			this.presence.PictureChanged += new EventHandler(this.presence_PictureChanged);
			this.presence.StatusChanged += new EventHandler(this.presence_StatusChanged);
			//this.presence.NimbusChanged += new EventHandler(this.presence_NimbusChanged);
			//this.presence.FocusChanged += new EventHandler(this.presence_FocusChanged);
			this.presence.NameChanged += new EventHandler(this.presence_NameChanged);

			string location = System.Reflection.Assembly.GetExecutingAssembly().Location;
			System.IO.DirectoryInfo dir = System.IO.Directory.GetParent(location);

			string onlineFile = System.IO.Path.Combine(dir.FullName, "./online.bmp");
			this.onlinePicture = Image.FromFile(onlineFile);

			string awayFile = System.IO.Path.Combine(dir.FullName, "./away.bmp");
			this.awayPicture = Image.FromFile(awayFile);

			string busyFile = System.IO.Path.Combine(dir.FullName, "./busy.bmp");
			this.busyPicture = Image.FromFile(busyFile);

			//show the name and the status and the message and a picture
			//show the video
			this.nameStatusPanel.Visible = true;
			this.nameStatusPanel.Dock = System.Windows.Forms.DockStyle.Top;
			this.nameStatusPanel.Height = 12;
			//this.statusPictureBox.Size = new Size(12,12);
			this.nameLbl.Text = this.presence.OwnerName;
			//Image i = this.statusPictureBox.Image;
			//this.statusPictureBox.Image = this.GetStatusImage();
//			if(null != i)
//			{
//				i.Dispose();
//			}

			this.messageLbl.Visible = true;
			this.messageLbl.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.messageLbl.Height = 25;
			this.messageLbl.Text = this.presence.Message;

			this.pictureBox.Visible = true;
			this.pictureBox.Dock = System.Windows.Forms.DockStyle.Fill;
			Image i = this.pictureBox.Image;
			this.pictureBox.Image = this.presence.GetImage(this.pictureBox.Size);
			if(null != i)
			{
				i.Dispose();
			}
			this.pictureBox.BringToFront();
		}

		/// <summary>
		/// Do setup after initialisation of all the dictionary stuff
		/// </summary>
		public void SetupTile()
		{
			if(!this.presence.Master && this.presence.IsDisplayAwareness)
			{
				// 
				// ownerAwarenessLbl
				// 
				this.ownerAwarenessLbl.Location = this.nameLbl.Location;
				this.ownerAwarenessLbl.Name = "ownerAwarenessLbl";
				this.ownerAwarenessLbl.TabIndex = 5;
				this.ownerAwarenessLbl.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
				this.Controls.Add(this.ownerAwarenessLbl);

				this.ownerAwarenessLbl.BackColor = this.presence.Colour.DarkColour;
				this.ownerAwarenessLbl.ForeColor = this.presence.Colour.BaseColour;

				this.presence.OwnerAwarenessChanged += new EventHandler(this.presence_OwnerAwarenessChanged);
				this.nameLbl.SizeChanged += new EventHandler(this.nameLbl_SizeChanged);
			}
		}
		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			this.presence.MessageChanged -= new EventHandler(this.presence_MessageChanged);
			this.presence.ImageChanged -= new EventHandler(this.presence_ImageChanged);
			this.presence.PictureChanged -= new EventHandler(this.presence_PictureChanged);
			this.presence.StatusChanged -= new EventHandler(this.presence_StatusChanged);
			//this.presence.NimbusChanged -= new EventHandler(this.presence_NimbusChanged);
			//this.presence.FocusChanged -= new EventHandler(this.presence_FocusChanged);
			this.presence.NameChanged -= new EventHandler(this.presence_NameChanged);

			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.nameLbl = new System.Windows.Forms.Label();
			this.messageLbl = new System.Windows.Forms.Label();
			this.pictureBox = new System.Windows.Forms.PictureBox();
			this.nameStatusPanel = new System.Windows.Forms.Panel();
			this.ownerAwarenessLbl = new System.Windows.Forms.Label();
			this.nameStatusPanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// nameLbl
			// 
			this.nameLbl.BackColor = System.Drawing.Color.SkyBlue;
			this.nameLbl.Dock = System.Windows.Forms.DockStyle.Fill;
			this.nameLbl.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.nameLbl.ForeColor = System.Drawing.Color.Black;
			this.nameLbl.Location = new System.Drawing.Point(0, 0);
			this.nameLbl.Name = "nameLbl";
			this.nameLbl.Size = new System.Drawing.Size(150, 10);
			this.nameLbl.TabIndex = 0;
			this.nameLbl.Text = "<name>";
			this.nameLbl.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// messageLbl
			// 
			this.messageLbl.BackColor = System.Drawing.Color.SkyBlue;
			this.messageLbl.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.messageLbl.ForeColor = System.Drawing.Color.Black;
			this.messageLbl.Location = new System.Drawing.Point(0, 80);
			this.messageLbl.Name = "messageLbl";
			this.messageLbl.Size = new System.Drawing.Size(100, 20);
			this.messageLbl.TabIndex = 1;
			this.messageLbl.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// pictureBox
			// 
			this.pictureBox.Location = new System.Drawing.Point(0, 20);
			this.pictureBox.Name = "pictureBox";
			this.pictureBox.Size = new System.Drawing.Size(100, 60);
			this.pictureBox.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage;
			this.pictureBox.TabIndex = 2;
			this.pictureBox.TabStop = false;
			this.pictureBox.Click += new System.EventHandler(this.pictureBox_Click);
			this.pictureBox.SizeChanged += new System.EventHandler(this.pictureBox_SizeChanged);
			// 
			// nameStatusPanel
			// 
			this.nameStatusPanel.Controls.Add(this.nameLbl);
			this.nameStatusPanel.Location = new System.Drawing.Point(0, 0);
			this.nameStatusPanel.Name = "nameStatusPanel";
			this.nameStatusPanel.Size = new System.Drawing.Size(150, 10);
			this.nameStatusPanel.TabIndex = 4;
			// 
			// PresenceTile
			// 
			this.BackColor = System.Drawing.Color.SkyBlue;
			this.Controls.Add(this.nameStatusPanel);
			this.Controls.Add(this.pictureBox);
			this.Controls.Add(this.messageLbl);
			this.Name = "PresenceTile";
			this.Size = new System.Drawing.Size(150, 100);
			this.nameStatusPanel.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		public int UserFocus
		{
			get
			{
				return this.focus;
			}
			set
			{
				this.focus = value;
				this.RefreshView();
			}
		}

		public int Nimbus
		{
			get
			{
				return this.nimbus;
			}
			set
			{
				this.nimbus = value;
				this.RefreshView();
			}
		}

		public int Awareness
		{
			get
			{
				int awareness = (int)((float)this.Nimbus*(float)this.UserFocus/100F);
				if(awareness < 5)
				{
					awareness = 5;
				}
				return awareness; 
			}
		}

		private void RefreshView()
		{
			this.SuspendLayout();
			this.Height = this.Awareness;
			if(AwarenessBetween(this.presence.StatusAwareness, this.presence.NameAwareness))
			{
				// blank, colour change on status change
				//this.nameStatusPanel.Visible = true;
				this.nameLbl.Text = String.Empty;
				//Image i = this.statusPictureBox.Image;
				//this.statusPictureBox.Image = null;
//				if(null != i)
//				{
//					i.Dispose();
//				}
				this.nameStatusPanel.Dock = System.Windows.Forms.DockStyle.Fill;

				this.messageLbl.Visible = false;
				this.pictureBox.Visible = false;
			}
			else if(AwarenessBetween(this.presence.NameAwareness, this.presence.MessageAwareness))
			{
				// enough room for the name to show
				this.nameStatusPanel.Visible = true;
				this.nameLbl.Text = this.presence.OwnerName;
				//Image i = this.statusPictureBox.Image;
				//this.statusPictureBox.Image = this.GetStatusImage();
//				if(null != i)
//				{
//					i.Dispose();
//				}
				//this.statusPictureBox.Size = new Size(this.Awareness, this.Awareness);
				this.nameStatusPanel.Dock = System.Windows.Forms.DockStyle.Fill;

				this.messageLbl.Visible = false;
				this.pictureBox.Visible = false;
			}
			else if(AwarenessBetween(this.presence.MessageAwareness, this.presence.PictureAwareness))
			{
				// enough room to show status as well now
				//this.nameStatusPanel.Visible = true;
				this.nameStatusPanel.Dock = System.Windows.Forms.DockStyle.Fill;
				this.nameStatusPanel.Height = 10;
				//this.statusPictureBox.Size = new Size(12,12);
				this.nameLbl.Text = this.presence.OwnerName;
				//this.statusPictureBox.Image = this.GetStatusImage();

				this.messageLbl.Visible = true;
				this.messageLbl.Dock = System.Windows.Forms.DockStyle.Bottom;
				this.messageLbl.Height = this.CalculateMessageHeight();
				this.messageLbl.Text = this.presence.Message;

				this.nameStatusPanel.BringToFront();

				this.pictureBox.Visible = false;
			}
			else if(AwarenessBetween(this.presence.PictureAwareness, this.presence.VideoAwareness))
			{
				// show the name and the status and the message and a picture
				this.nameStatusPanel.Visible = true;
				this.nameStatusPanel.Dock = System.Windows.Forms.DockStyle.Top;
				this.nameStatusPanel.Height = 10;
				//this.statusPictureBox.Size = new Size(12,12);
				//this.nameLbl.Text = this.presence.OwnerName;
				//this.statusPictureBox.Image = this.GetStatusImage();

				this.messageLbl.Visible = true;
				this.messageLbl.Dock = System.Windows.Forms.DockStyle.Bottom;
				this.messageLbl.Height = this.CalculateMessageHeight();
				//this.messageLbl.Text = this.presence.Message;

				this.pictureBox.Visible = true;
				this.pictureBox.Dock = System.Windows.Forms.DockStyle.Fill;
				//Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetPicture(this.pictureBox.Size);
//				if(null != i)
//				{
//					i.Dispose();
//				}
				this.pictureBox.BringToFront();
			}
			else
			{
				// show the name and the status and the message and a picture
				// show the video
				this.nameStatusPanel.Visible = true;
				this.nameStatusPanel.Dock = System.Windows.Forms.DockStyle.Top;
//				this.nameStatusPanel.Height = 12;
//				this.statusPictureBox.Size = new Size(12,12);
//				this.nameLbl.Text = this.presence.OwnerName;
//				this.statusPictureBox.Image = this.GetStatusImage();

				this.messageLbl.Visible = true;
//				this.messageLbl.Dock = System.Windows.Forms.DockStyle.Bottom;
				this.messageLbl.Height = this.CalculateMessageHeight();
//				this.messageLbl.Text = this.presence.Message;

				this.pictureBox.Visible = true;
				//this.pictureBox.Dock = System.Windows.Forms.DockStyle.Fill;
				Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetImage(this.pictureBox.Size);
				if(null != i)
				{
					i.Dispose();
				}
				this.pictureBox.BringToFront();
			}
			if(this.presence.IsDisplayAwareness && !this.presence.Master)
			{
				this.ownerAwarenessLbl.BringToFront();
			}
			this.ResumeLayout(true);
		}

		private int CalculateMessageHeight()
		{
			int numChars = this.presence.Message.Length;
			int maxPixelLength = numChars*7+(numChars-1);
			int lines = 0;
			if(maxPixelLength > this.Width)
			{
				lines = 2;
			}
			else if(maxPixelLength > 0)
			{
				lines = 1;
			}
			return lines*10;
		}

		/// <summary>
		/// Returns true if the focus is between lower and higher.
		/// 
		/// Range includes lower but not higher.
		/// </summary>
		/// <param name="lower">Lower bound of comparison</param>
		/// <param name="higher">Higher bound of comparison</param>
		/// <returns>True when focus is between lower and higher.</returns>
		private bool AwarenessBetween(int lower, int higher)
		{
			return ((this.Awareness >= lower) && (this.Awareness < higher));
		}

		private void presence_MessageChanged(object sender, EventArgs e)
		{
			this.messageLbl.Text = this.presence.Message;
			this.messageLbl.Height = this.CalculateMessageHeight();
		}

		private void presence_ImageChanged(object sender, EventArgs e)
		{
			if(this.Awareness >= 90)
			{
				Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetImage(this.pictureBox.Size);
				if(null != i)
				{
					i.Dispose();
				}
			}
		}

		private void presence_PictureChanged(object sender, EventArgs e)
		{
			if(this.AwarenessBetween(60, 90))
			{
				//Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetPicture(this.pictureBox.Size);
//				if(null != i)
//				{
//					i.Dispose();
//				}
			}
		}

		private void presence_StatusChanged(object sender, EventArgs e)
		{
			//Image i = this.statusPictureBox.Image;
			//this.statusPictureBox.Image = this.GetStatusImage();
//			if(null != i)
//			{
//				i.Dispose();
//			}
			if(this.presence.Status != "Online")
			{
				this.nameLbl.BackColor = this.presence.Colour.AwayColour;
				this.messageLbl.BackColor = this.presence.Colour.AwayColour;
				this.pictureBox.BackColor = this.presence.Colour.AwayColour;
				this.nameStatusPanel.BackColor = this.presence.Colour.AwayColour;
			}
			else if(this.presence.Status == "Online")
			{
				this.OnlineColour();
			}
			if(!this.presence.Master && this.presence.IsDisplayAwareness)
			{
				this.ownerAwarenessLbl.Visible = (this.presence.Status == "Online");
			}
		}

		public void OnlineColour()
		{
			this.nameLbl.BackColor = this.presence.Colour.PresenceColour;
			this.messageLbl.BackColor = this.presence.Colour.PresenceColour;
			this.pictureBox.BackColor = this.presence.Colour.PresenceColour;
			this.nameStatusPanel.BackColor = this.presence.Colour.PresenceColour;
		}

		private Image GetStatusImage()
		{
			Image i;
			switch(this.presence.Status)
			{
				case "Online":
					i = this.onlinePicture;
					break;
				case "On the Phone":
				case "Busy":
					i = this.busyPicture;
					break;
				default:
					i = this.awayPicture;
					break;
			}
			return i;
		}

		private void presence_NimbusChanged(object sender, EventArgs e)
		{
			this.RefreshView();
		}

		private void pictureBox_SizeChanged(object sender, System.EventArgs e)
		{
			if(this.AwarenessBetween(90, 101))
			{
				Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetImage(this.pictureBox.Size);
				if(null != i)
				{
					i.Dispose();
				}
			}
			else if(this.AwarenessBetween(60, 90))
			{
				//Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetPicture(this.pictureBox.Size);
//				if(null != i)
//				{
//					i.Dispose();
//				}
			}
		}

		private void presence_FocusChanged(object sender, EventArgs e)
		{
			this.RefreshView();
		}

		private void pictureBox_Click(object sender, System.EventArgs e)
		{
			if(this.presence.Master && this.Awareness >= 90)
			{
				this.presence.UpdateCamera();
			}
		}

		private void presence_NameChanged(object sender, EventArgs e)
		{
			this.nameLbl.Text = this.presence.OwnerName;
		}

		private void presence_OwnerAwarenessChanged(object sender, EventArgs e)
		{
			this.UpdateOwnerAwarenessLbl();
		}

		private void UpdateOwnerAwarenessLbl()
		{
			this.ownerAwarenessLbl.Text = this.nameLbl.Text;

			try
			{
				int ownerAwareness = this.presence.OwnerAwareness;
				int totalWidth = this.nameLbl.Width;
				int displayWidth = (int)((float)ownerAwareness/100F*totalWidth);
				this.ownerAwarenessLbl.Width = displayWidth;
			}
			catch(Exception e)
			{
				this.ownerAwarenessLbl.Width = this.nameLbl.Width;
			}

			this.ownerAwarenessLbl.Height = this.nameLbl.Height;

			this.ownerAwarenessLbl.BringToFront();
		}

		private void nameLbl_SizeChanged(object sender, EventArgs e)
		{
			this.UpdateOwnerAwarenessLbl();
		}
	}
}
