using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace PresenceItem
{
	/// <summary>
	/// Summary description for SlaveTransient.
	/// </summary>
	public class SlaveTransient : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.Label statusLbl;
		private System.Windows.Forms.Button inviteBtn;
		private System.Windows.Forms.Label messageLbl;
		private System.Windows.Forms.PictureBox pictureBox;
		private System.Windows.Forms.Label pictureLbl;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Label nameLbl;
		private System.Windows.Forms.Label ownerAwarenessNmLbl;
		private System.Windows.Forms.Label ownerAwarenessStLbl;

		private PresenceItem presence;

		public SlaveTransient(PresenceItem presence)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			this.presence = presence;

			// do the colours
			this.inviteBtn.BackColor = this.presence.Colour.DarkColour;
			this.inviteBtn.ForeColor = this.presence.Colour.BaseColour;
			this.BackColor = this.presence.Colour.PresenceColour;

			//this.pictureBox.Enabled = true;
			this.pictureLbl.Visible = false;
			this.nameLbl.Text = this.presence.OwnerName;
			this.statusLbl.Text = this.presence.Status;
			this.messageLbl.Text = this.presence.Message;
			Image i = this.pictureBox.Image;
			this.pictureBox.Image = this.presence.GetImage(this.pictureBox.Size);
			if(null != i)
			{
				i.Dispose();
			}

			// do the owner awareness stuff
			if(this.presence.IsDisplayAwareness)
			{
				this.ownerAwarenessNmLbl = new System.Windows.Forms.Label();
				this.ownerAwarenessStLbl = new System.Windows.Forms.Label();
				// 
				// ownerAwarenessNmLbl
				// 
				this.ownerAwarenessNmLbl.Location = this.nameLbl.Location;
				this.ownerAwarenessNmLbl.Size = new Size(this.nameLbl.Width, this.nameLbl.Height);
				this.ownerAwarenessNmLbl.Name = "ownerAwarenessNmLbl";
				this.ownerAwarenessNmLbl.TabIndex = 6;
				this.ownerAwarenessStLbl.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
				this.ownerAwarenessNmLbl.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
				this.ownerAwarenessNmLbl.Text = this.nameLbl.Text;
				this.ownerAwarenessNmLbl.BackColor = this.presence.Colour.DarkColour;
				this.ownerAwarenessNmLbl.ForeColor = this.presence.Colour.BaseColour;
				this.ownerAwarenessNmLbl.BringToFront();
				// 
				// ownerAwarenessStLbl
				// 
				this.ownerAwarenessStLbl.Location = this.statusLbl.Location;
				this.ownerAwarenessStLbl.Size = new Size(this.statusLbl.Width, this.statusLbl.Height);
				this.ownerAwarenessStLbl.Name = "ownerAwarenessStLbl";
				this.ownerAwarenessStLbl.TabIndex = 7;
				this.ownerAwarenessStLbl.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
				this.ownerAwarenessStLbl.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
				this.ownerAwarenessStLbl.Text = this.statusLbl.Text;
				this.ownerAwarenessStLbl.BackColor = this.presence.Colour.DarkColour;
				this.ownerAwarenessStLbl.ForeColor = this.presence.Colour.BaseColour;
				this.ownerAwarenessStLbl.BringToFront();

				this.Controls.Add(this.ownerAwarenessStLbl);
				this.Controls.Add(this.ownerAwarenessNmLbl);

				this.presence.OwnerAwarenessChanged += new EventHandler(this.presence_OwnerAwarenessChanged);
			}

			this.presence.NimbusChanged += new EventHandler(this.presence_NimbusChanged);
			this.presence.StatusChanged += new EventHandler(this.presence_StatusChanged);
			this.presence.MessageChanged += new EventHandler(this.presence_MessageChanged);
			this.presence.PictureChanged += new EventHandler(this.presence_PictureChanged);
			this.presence.ImageChanged += new EventHandler(this.presence_ImageChanged);

			this.presence_StatusChanged(this.presence, new EventArgs());
			this.RefreshView();
		}


		private void RefreshView()
		{
			if(this.presence.Nimbus >= this.presence.VideoAwareness)
			{
				//this.pictureBox.Enabled = true;
				//this.pictureLbl.Visible = false;
				Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetImage(this.pictureBox.Size);
				if(null != i)
				{
					i.Dispose();
				}
			}
			else if(this.presence.Nimbus >= this.presence.PictureAwareness)
			{
				//this.pictureBox.Enabled = true;
				this.pictureLbl.Visible = false;
				//Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetPicture(this.pictureBox.Size);
//				if(null != i)
//				{
//					i.Dispose();
//				}
			}
			else
			{
				//this.pictureBox.Enabled = false;
				this.pictureBox.Image = null;
				this.pictureLbl.Visible = true;
			}
			if(this.presence.Nimbus >= this.presence.MessageAwareness)
			{
				//this.messageLbl.Enabled = true;
				this.messageLbl.Text = this.presence.Message;
			}
			else
			{
				//this.messageLbl.Enabled = false;
				this.messageLbl.Text = string.Empty;
			}
			if(this.presence.Nimbus >= this.presence.StatusAwareness)
			{
				//this.statusLbl.Enabled = true;
				this.statusLbl.Text = this.presence.Status;
			}
			else
			{
				this.statusLbl.Text = string.Empty;
				//this.statusLbl.Enabled = false;
			}
			if(this.presence.IsDisplayAwareness)
			{
				this.UpdateOwnerAwareness();
			}
		}
		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			this.presence.NimbusChanged -= new EventHandler(this.presence_NimbusChanged);
			this.presence.StatusChanged -= new EventHandler(this.presence_StatusChanged);
			this.presence.MessageChanged -= new EventHandler(this.presence_MessageChanged);
			this.presence.PictureChanged -= new EventHandler(this.presence_PictureChanged);
			this.presence.ImageChanged -= new EventHandler(this.presence_ImageChanged);

			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.statusLbl = new System.Windows.Forms.Label();
			this.inviteBtn = new System.Windows.Forms.Button();
			this.messageLbl = new System.Windows.Forms.Label();
			this.pictureBox = new System.Windows.Forms.PictureBox();
			this.pictureLbl = new System.Windows.Forms.Label();
			this.nameLbl = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// statusLbl
			// 
			this.statusLbl.BackColor = System.Drawing.Color.SkyBlue;
			this.statusLbl.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.statusLbl.ForeColor = System.Drawing.Color.Black;
			this.statusLbl.Location = new System.Drawing.Point(80, 0);
			this.statusLbl.Name = "statusLbl";
			this.statusLbl.Size = new System.Drawing.Size(70, 12);
			this.statusLbl.TabIndex = 0;
			this.statusLbl.Text = "<status>";
			this.statusLbl.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// inviteBtn
			// 
			this.inviteBtn.BackColor = System.Drawing.Color.Navy;
			this.inviteBtn.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
			this.inviteBtn.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.inviteBtn.ForeColor = System.Drawing.Color.SkyBlue;
			this.inviteBtn.Location = new System.Drawing.Point(96, 120);
			this.inviteBtn.Name = "inviteBtn";
			this.inviteBtn.Size = new System.Drawing.Size(56, 30);
			this.inviteBtn.TabIndex = 1;
			this.inviteBtn.Text = "Invite to Place";
			this.inviteBtn.Click += new System.EventHandler(this.inviteBtn_Click);
			// 
			// messageLbl
			// 
			this.messageLbl.BackColor = System.Drawing.Color.SkyBlue;
			this.messageLbl.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.messageLbl.ForeColor = System.Drawing.Color.Black;
			this.messageLbl.Location = new System.Drawing.Point(0, 120);
			this.messageLbl.Name = "messageLbl";
			this.messageLbl.Size = new System.Drawing.Size(96, 30);
			this.messageLbl.TabIndex = 2;
			this.messageLbl.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// pictureBox
			// 
			this.pictureBox.Location = new System.Drawing.Point(0, 12);
			this.pictureBox.Name = "pictureBox";
			this.pictureBox.Size = new System.Drawing.Size(150, 108);
			this.pictureBox.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage;
			this.pictureBox.TabIndex = 3;
			this.pictureBox.TabStop = false;
			// 
			// pictureLbl
			// 
			this.pictureLbl.Font = new System.Drawing.Font("Tahoma", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.pictureLbl.Location = new System.Drawing.Point(16, 48);
			this.pictureLbl.Name = "pictureLbl";
			this.pictureLbl.Size = new System.Drawing.Size(112, 23);
			this.pictureLbl.TabIndex = 4;
			this.pictureLbl.Text = "No Picture Available";
			this.pictureLbl.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// nameLbl
			// 
			this.nameLbl.Location = new System.Drawing.Point(0, 0);
			this.nameLbl.Name = "nameLbl";
			this.nameLbl.Size = new System.Drawing.Size(80, 12);
			this.nameLbl.TabIndex = 5;
			this.nameLbl.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// SlaveTransient
			// 
			this.BackColor = System.Drawing.Color.SkyBlue;
			this.Controls.Add(this.nameLbl);
			this.Controls.Add(this.pictureLbl);
			this.Controls.Add(this.pictureBox);
			this.Controls.Add(this.messageLbl);
			this.Controls.Add(this.inviteBtn);
			this.Controls.Add(this.statusLbl);
			this.Name = "SlaveTransient";
			this.ResumeLayout(false);

		}
		#endregion

		private void presence_NimbusChanged(object sender, EventArgs e)
		{
			this.RefreshView();
		}

		private void presence_StatusChanged(object sender, EventArgs e)
		{
			if(this.presence.Nimbus >= this.presence.StatusAwareness)
			{
				this.statusLbl.Text = this.presence.Status;
			}
			if(this.presence.Status != "Online")
			{
				this.nameLbl.BackColor = this.presence.Colour.AwayColour;
				this.pictureLbl.BackColor = this.presence.Colour.AwayColour;
				this.pictureBox.BackColor = this.presence.Colour.AwayColour;
				this.statusLbl.BackColor = this.presence.Colour.AwayColour;
				this.messageLbl.BackColor = this.presence.Colour.AwayColour;
			}
			else if(this.presence.Status == "Online")
			{
				this.nameLbl.BackColor = this.presence.Colour.PresenceColour;
				this.pictureLbl.BackColor = this.presence.Colour.PresenceColour;
				this.pictureBox.BackColor = this.presence.Colour.PresenceColour;
				this.statusLbl.BackColor = this.presence.Colour.PresenceColour;
				this.messageLbl.BackColor = this.presence.Colour.PresenceColour;
			}
		}

		private void presence_MessageChanged(object sender, EventArgs e)
		{
			if(this.presence.Nimbus >= this.presence.MessageAwareness)
			{
				this.messageLbl.Text = this.presence.Message;
			}
		}

		private void presence_PictureChanged(object sender, EventArgs e)
		{
			if(this.presence.Nimbus >= this.presence.PictureAwareness && this.presence.Nimbus < this.presence.VideoAwareness)
			{
				//Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetPicture(this.pictureBox.Size);
//				if(null != i)
//				{
//					i.Dispose();
//				}
			}
		}

		private void presence_ImageChanged(object sender, EventArgs e)
		{
			if(this.presence.Nimbus >= this.presence.VideoAwareness)
			{
				Image i = this.pictureBox.Image;
				this.pictureBox.Image = this.presence.GetImage(this.pictureBox.Size);
				if(null != i)
				{
					i.Dispose();
				}
			}
		}

		private void inviteBtn_Click(object sender, System.EventArgs e)
		{
			this.presence.ShowInvite();
		}

		private void presence_OwnerAwarenessChanged(object sender, EventArgs e)
		{
			this.UpdateOwnerAwareness();
		}

		private void UpdateOwnerAwareness()
		{
			int fullWidth = this.nameLbl.Width + this.statusLbl.Width;
			int awarenessWidth = (int)((float)this.presence.OwnerAwareness/100F*(float)fullWidth);
			if(awarenessWidth > this.nameLbl.Width)
			{
				this.ownerAwarenessNmLbl.Width = this.nameLbl.Width;
				this.ownerAwarenessStLbl.Visible = true;
				this.ownerAwarenessStLbl.Width = awarenessWidth - this.nameLbl.Width;
			}
			else
			{
				this.ownerAwarenessStLbl.Visible = false;
				this.ownerAwarenessNmLbl.Width = awarenessWidth;
			}
			this.ownerAwarenessNmLbl.BringToFront();
			this.ownerAwarenessStLbl.BringToFront();
		}
	}
}
